;**** Processor Definition
.include "2313def.inc"

;*****************************************************************************************
;	Variablen
;*****************************************************************************************

.def		NibConv1  = r1					; registers used for nibble conversion
.def		NibConv2  = r2
.def		NibConv3  = r3
.def		NibConv4  = r4
.def		NibConv5  = r5
.def		ConvRes1  = r6					; result registers for multiplikation
.def		ConvRes2  = r7
.def		ConvRes3  = r8
.def		ConvRes4  = r9
.def		ConvRes5  = r10
.def		TempLow3  = r11
.def		Temp	  = r16					; temporary register
.def		Temp2	  = r17					; temporary register
.def		Status1	  = r18					; status register
											; bit 0 1=transmission from caliper running
											; bit 1 1=edge measure
											; bit 2 1=start conversion
											; bit 3 1=conversion finished, update send registers
											; bit 4 1=UART transmission running
											; bit 5 1= negative value in caliper
.def		NibNr	  = r19					; Number of read Nibbles
.def		MulDiv1   = r20					; store multipliers and divisors
.def		MulDiv2   = r21
.def		MulDiv3   = r22
.def		MulDiv4   = r23
.def		MulDiv5   = r24
.def		DelayTemp = r25
; ZL is used to adress Send Digit

;*****************************************************************************************
;	SRAM-Variablen
;*****************************************************************************************

.equ	Nibble12		= 0x60				; 1st and 2nd caliper nibble
.equ	Nibble34		= 0x61				; 3rd and 4th caliper nibble
.equ	Nibble5			= 0x62				; 5th nibble
.equ	SendNib1		= 0x63				; 1 Nibble is 4 Bits of conversion result
.equ	SendNib2		= 0x64
.equ	SendNib3		= 0x65
.equ	SendNib4		= 0x66				; highest Nibble
.equ	SendNib5		= 0x67
.equ	SendNib6		= 0x68
.equ	ToSendNib1		= 0x69				; used between conversion and send
.equ	ToSendNib2		= 0x6A
.equ	ToSendNib3		= 0x6B
.equ	ToSendNib4		= 0x6C
.equ	ToSendNib5		= 0x6D
.equ	ToSendNib6		= 0x6E

;*****************************************************************************************
;	Werte
;*****************************************************************************************

.equ	number		= 1						; adress number
.equ	avradress	= 0x80+number			; avr address for receiving/transmitting
.equ	avrzero		= 0x90+number			; send zero signal
.equ	avrstate	= 0xA0+number			; send state signal
.equ	avrfast		= 0xB0+number			; enter fast mode
;.equ	avrnormal	= 0xC4					; enter normal mode

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

		rjmp	START						; reset vector
		rjmp	INT0ROUTINE					; Int0 Pin
		reti								; Int1 Pin
		reti								; Timer/Counter1 Capture
		reti								; Timer/Counter1 Compare A
		reti								; Timer/Counter1 Overflow
		reti								; Timer/Counter0 Overflow
		rjmp	RXCROUTINE					; USART0 Rx complete
		rjmp	UDREROUTINE					; USART0, UDRE
		reti								; USART0 Tx complete
		reti								; analog comparator
		reti								; Pin change interrupt
		reti								; Timer/Counter1 Compare B
		reti								; Timer/Counter0 Compare A
		reti								; Timer/Counter0 Compare B
		reti								; USI start condition
		reti								; USI overflow
		reti								; Eeprom ready
		reti								; Watchdog Timer overflow

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************
START:
		ldi		Temp, low(RAMEND)			; initialise stack pointer
		out		spl, Temp
		cbi		DDRD, 2						; set pins PD2 and PD3 (int0, int1) to input
		cbi		DDRD, 3
		cbi		PORTD, 4
		cbi		PORTD, 5
		sbi		DDRD, 4						; set the caliper control pins
		sbi		DDRD, 5
		sbi		PORTD, 2					; pull the pins weak high
		sbi		PORTD, 3
		clr		Temp						; disable watchdog
		out		WDTCR, Temp					; wdif, wdie, wdp3, wdce, wde, wdp2..0
		ldi		Temp, 0b00000010			; disable sleep modes and set ext interrupt1
											; to rising edge (falling plus inverter)
		out		MCUCR, Temp					; PUD, SM1, SE, SM0, ISC11, 10, 01, 00
		ldi		Temp, 0b01000000			; enable interrupt 0
		out		GIMSK, Temp					; INT1, INT0, PCIE
		ldi		Temp, 0b00000000			; set normal count mode and clock prescaler
		out		TCCR0, Temp					; , , , , , cs02, cs01, cs00
											; stop clock
											; TCNT0, Timer Register, set in routine
		ldi		Temp, 0b00000000			; TOIE1, OCIE1A, OCIE1B, , ICIE1, OCIE0B, OCIE0A
		out		TIMSK, Temp					; disable Timer 0 overflow interrupt
		ldi		Temp, 72;36					; set Baud rate to 19200@8MHz, U2X=0=UMSEL
		out		UBRR, Temp
		ldi		Temp, (1<<RXEN)|(1<<rxcie)	;|(1<<TXEN)	; enable Rx and Tx pins
		out 	UCR, Temp					; rxcie, txcie, udrie, rxen, txen, ucsz2, rxb8, txb8
		clr		Status1						; set initial status
		;rcall	DELAY10s					; wait for the caliper
		;rcall	CALIPER_ZERO				; set it to zero
		;rcall	DELAY10s					; wait again
		;rcall	CALIPER_FAST				; go into the fast mode
		sei									; enable interrupts
MAIN:
		sbrc	Status1, 2					; start conversion if caliper transmission finished
		rcall	CONVERSION
		sbrc	Status1, 3
		rcall	UPDATE_SEND					; update send registers if conversion finished
		rjmp	MAIN

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

INT0ROUTINE:						; waits for synchronisation pulse (>95us high)
											; starts transfer from caliper if pulse occured
											; 20 bit data are stored in Nibble1..5
		push	Temp
		push	Temp2
		mov		Temp, Status1
		andi	Temp, 0b00000011
		cpi		Temp, 1						; check for running transfer
		breq	INT0ROUTINE_READ_DATA
		brlo	INT0ROUTINE_START_PULSE_MEAS
		rjmp	INT0ROUTINE_STOP_PULSE_MEAS
 INT0ROUTINE_START_PULSE_MEAS:
 		ldi		Temp, 0b00000010
		out		TCCR0, Temp					; start the timer clk/8 prescaler
		in		Temp, MCUCR
		ori		Temp, 0b000000001
		out		MCUCR, Temp					; set int1 to rising edge
		ori		Status1, 0b00000010			; set edge measure state
		rjmp	INT0ROUTINE_OUT				; go out
 INT0ROUTINE_STOP_PULSE_MEAS:
 		in		Temp, TCNT0					; read the timer value
		clr		Temp2
		out		TCCR0, Temp2				; stop timer
		out		TCNT0, Temp2				; reset timer
		andi	Status1, 0b11111101			; reset sync detection state
		cpi		Temp, 188;94					; >94us
		brsh	INT0ROUTINE_SYNC_DETECTED	; pulse occured, start transfer state
		in		Temp, MCUCR					; set int to falling edge
		andi	Temp, 0b11111110
		out		MCUCR, Temp
		rjmp	INT0ROUTINE_OUT				; go out
 INT0ROUTINE_SYNC_DETECTED:
 		ori		Status1, 0b00000001			; set transmission state
		clr		NibNr						; set nr of received pulses 0
		sts		Nibble12, NibNr				; clear Nibbles
		sts		Nibble34, NibNr
		sts		Nibble5, NibNr	
		rjmp	INT0ROUTINE_OUT				; and skip 1st pulse
 INT0ROUTINE_READ_DATA:
		in		Temp2, PIND					; read port
 		inc		NibNr						; increase NibNr
		cpi		NibNr, 24					; long pulse is last pulse
		brsh	INT0ROUTINE_LASTNIB
		cpi		NibNr, 23					; ignore bit 24
		brsh	INT0ROUTINE_OUT
 		cpi		NibNr, 19					; Nibble5	
		brsh	INT0ROUTINE_NIB5
		cpi		NibNr, 11					; Nibbles3,4
		brsh	INT0ROUTINE_NIB34
		cpi		NibNr, 3					; Nibbles1,2
		brsh	INT0ROUTINE_NIB12
		rjmp	INT0ROUTINE_OUT				; skip 2nd and 3rd pulse
 INT0ROUTINE_LASTNIB:
 		andi	Status1, 0b11111110			; clear transmit state
		ori		Status1, 0b00000100			; set start conversion state
		in		Temp, MCUCR
		andi	Temp, 0b11111011
		out		MCUCR, Temp					; set int to falling edge
		rjmp	INT0ROUTINE_OUT
 INT0ROUTINE_NIB34:
 		lds		Temp, Nibble34				; load
		lsr		Temp						; shift right
		sbrc	Temp2, 3					; set bit if pin set (high active)
		ori		Temp, 0b10000000
		sts		Nibble34, Temp				; store
		rjmp	INT0ROUTINE_OUT				; go out
 INT0ROUTINE_NIB5:
 		lds		Temp, Nibble5 				; load
		lsr		Temp						; shift right
		sbrc	Temp2, 3					; set bit if pin set (high active)
		ori		Temp, 0b00001000
		sts		Nibble5, Temp				; store
		rjmp	INT0ROUTINE_OUT				; go out
 INT0ROUTINE_NIB12:
 		lds		Temp, Nibble12				; load
		lsr		Temp						; shift right
		sbrc	Temp2, 3					; set bit if pin set (high active)
		ori		Temp, 0b10000000
		sts		Nibble12, Temp				; store
 INT0ROUTINE_OUT:
 		pop		Temp2
		pop		Temp
		reti

RXCROUTINE:							; tests if adressed and inits send the stored caliper value
		push	Temp
		push	Temp2
		in		Temp2, USR					; read state register
		in		Temp, UDR					; read the value
		andi	Temp2, (1<<FE)|(1<<3)
		brne	RXCROUTINE_OUT				; go out if error occured
		cpi		Temp, avradress				; verify sent adress
		breq	RXCROUTINE_START_SEND
		cpi		Temp, avrstate
		breq	RXCROUTINE_STATE
		cpi		Temp, avrzero
		breq	RXCROUTINE_ZERO
		cpi		Temp, avrfast
		breq	RXCROUTINE_FAST
		rjmp	RXCROUTINE_OUT				; go out if not adressed
 RXCROUTINE_START_SEND:
		sbi		UCR, TXEN					; enable TX pin
		ori		Status1, 0b00010000			; set transmit state
		ldi		ZL, SendNib1				; set SendDigit to first send adress
		sbi		UCR, UDRIE					; enable UDRE interrupt	
		rjmp	RXCROUTINE_OUT
 RXCROUTINE_STATE:
		rcall	CALIPER_STATE
		rjmp	RXCROUTINE_OUT
 RXCROUTINE_ZERO:
		rcall	CALIPER_ZERO
		rjmp	RXCROUTINE_OUT
 RXCROUTINE_FAST:
 		rcall	CALIPER_FAST
 RXCROUTINE_OUT:
 		pop		Temp2
		pop		Temp
		reti

UDREROUTINE:						; sends the stored caliper value
		push	Temp
		clr		ZH
		ld		Temp, Z+					; load from adress and increment
		out		UDR, Temp					; send value
		cpi		ZL, ToSendNib1				; 1st adress after SendNib6
		brlo	UDREROUTINE_OUT				; stop if last adress sent
		andi	Status1, 0b11101111			; reset transfer running state
		cbi		UCR, UDRIE					; disable UDR empty interrupt
		cbi		UCR, TXEN					; disable TX pin after transmission

 UDREROUTINE_OUT:
		pop		Temp
		reti

;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************

CONVERSION:							; convert the caliper value (divide by 1,00787) and store result in
											; ToSendNib1..6
		cli									; disable interrupts
		push	Temp
		lds		NibConv1, Nibble12			; move stored nibbles
		lds		NibConv2, Nibble34
		lds		NibConv3, Nibble5
		sei									; enable interrupts
		andi	Status1, 0b11110011			; reset start conversion and conversion finished state
		clr		NibConv4
		clr		NibConv5
		clr		ConvRes1					; clear result registers
		clr		ConvRes2
		clr		ConvRes3
		clr		ConvRes4
		clr		ConvRes5
		andi	Status1, 0b11011111			; first reset negative state
		sbrs	NibConv3, 3					; test if negative value
		rjmp	CONVERSION_POSITIVE			; no, normal run
		ori		Status1, 0b00100000			; set negative value state
		ldi		MulDiv1, 0xFF				; first invert all bits
		eor		NibConv1, MulDiv1
		eor		NibConv2, MulDiv1
		ldi		MulDiv1, 0b00001111			; complement last 4 bits (5th nibble)
		eor		NibConv3, MulDiv1
		ldi		MulDiv1, 1					; then increase the absolute value
		add		NibConv1, MulDiv1
		clr		MulDiv1
		adc		NibConv2, MulDiv1
		adc		NibConv3, MulDiv1
 CONVERSION_POSITIVE:
		; first multiply by 100000 (h186A0 b11000011010100 )
		; then divide by 100787    (h189b3 b)
		ldi		MulDiv1, 0xA0				; load multiplier
		ldi		MulDiv2, 0x86
		ldi		MulDiv3, 0x01
		rcall	MULTIPLY					; multiply by 100000
		rcall	DIVIDE						; divide by 100787
		rcall	MAKE_SEND_REGISTERS
		andi	Status1, 0b11111011			; reset start conversion bit
		ori		Status1, 0b00001000			; set conversion finished state
		pop		Temp
		ret

UPDATE_SEND:						; update send registers if conversion finished and
											; no send routine active
		push	Temp
		in		Temp, UCR					; save UCSRB state
		cbi		UCR, RXCIE					; disable RXCIE
		sbrc	Status1, 4					; dont update during transmission
		rjmp	UPDATE_NO
		push	Temp2
		lds		Temp2, ToSendNib1			; update the send registers
		sts		SendNib1, Temp2
		lds		Temp2, ToSendNib2
		sts		SendNib2, Temp2
		lds		Temp2, ToSendNib3
		sts		SendNib3, Temp2
		lds		Temp2, ToSendNib4
		sts		SendNib4, Temp2
		lds		Temp2, ToSendNib5
		sts		SendNib5, Temp2
		lds		Temp2, ToSendNib6
		sts		SendNib6, Temp2
		andi	Status1, 0b11110111			; reset convertion finished state
		pop		Temp2
 UPDATE_NO:
 		out		UCR, Temp					; restore UCSRB state
 		pop		Temp
		ret

MULTIPLY:							; multiplies ConvNib1..5 with 17 bits of MulDiv1..3
											; result is stored in ConvRes1..5
											; ConvNib is NOT saved!!
		push	Temp
		ldi		Temp, 17					; multpplier is 17 bit
 MULTIPLY_INNER:							;loop
 		sbrs	MulDiv1, 0					; test if bit 1
		rjmp	MULTIPLY_NO					; no, dont multiply
 		add		ConvRes1, NibConv1
		adc		ConvRes2, NibConv2
		adc		ConvRes3, NibConv3
		adc		ConvRes4, NibConv4
		adc		ConvRes5, NibConv5
 MULTIPLY_NO:
 		lsl		NibConv1					; prepare next step
		rol		NibConv2					; shift count
		rol		NibConv3
		rol		NibConv4
		rol		NibConv5
		lsr		MulDiv3						; shift multpplier
		ror		MulDiv2
		ror		MulDiv1
		dec		Temp						; decrease bit counter
		brne	MULTIPLY_INNER				; go on if bits there
		pop		Temp
		ret

DIVIDE:								; divide ConvRes1..5 by 17 bits of MulDiv1..3
		push	Temp
		push	Temp2
		ldi		MulDiv5, 0x0C				; this is already the shifted divisor!
		ldi		MulDiv4, 0x4D
		ldi		MulDiv3, 0x98
		clr		MulDiv2
		clr		MulDiv1
		lsr		MulDiv5
		ror		MulDiv4
		ror		MulDiv3
		clr		NibConv1					; clear result registers
		clr		NibConv2
		clr		NibConv3
		clr		NibConv4
		clr		NibConv5
		ldi		Temp, 19					; 36-17=19 counts of shifts until original
											; divisor achieved
 DIVIDE_INNER:								; inner dividion loop
 		lsl		NibConv1					; shift result register 1 left
		rol		NibConv2
		rol		NibConv3					; 3 registers are enough for 19 shifts
		ori		Temp2, 0b00000001			; first set last bit
 		sub		ConvRes1, MulDiv1			; compare the counts
		sbc		ConvRes2, MulDiv2
		sbc		ConvRes3, MulDiv3
		sbc		ConvRes4, MulDiv4
		sbc		ConvRes5, MulDiv5
		brsh	DIVIDE_INNER_NOUNDO			; dont restore value if carry not set
		andi	Temp2, 0b00000000			; reset last bit, not divided
		add		ConvRes1, MulDiv1			; undo subtraction
		adc		ConvRes2, MulDiv2
		adc		ConvRes3, MulDiv3
		adc		ConvRes4, MulDiv4
		adc		ConvRes5, MulDiv5
 DIVIDE_INNER_NOUNDO:
 		or		NibConv1, Temp2				; add the resulting lsb
 		lsr		MulDiv5						; shift divisor 1 right
		ror		MulDiv4
		ror		MulDiv3
		ror		MulDiv2
		ror		MulDiv1
		dec		Temp						; decrease loop counter
		brsh	DIVIDE_INNER				; run until counter zero
		pop		Temp2
		pop		Temp
		ret

MAKE_SEND_REGISTERS:				; convert the result to 4bit resigters
		push	Temp
		sbrc	Status1, 5					; set negative sign if number negative
		rjmp	MAKE_SEND_REG_NEG			; routine for negative result
		mov		Temp, NibConv1
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib1, Temp			; store it
		mov		Temp, NibConv1
		swap	Temp						; only use higher nibble
		andi	Temp, 0b00001111
		sts		ToSendNib2, Temp			; store it
		mov		Temp, NibConv2
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib3, Temp			; store it
		mov		Temp, NibConv2
		swap	Temp						; only use higher nibble
		andi	Temp, 0b00001111
		sts		ToSendNib4, Temp			; store it
		mov		Temp, NibConv3
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib5, Temp			; store it
		mov		Temp, NibConv3
		swap	Temp						; only use higher nibble
		andi	Temp, 0b00001111
		sts		ToSendNib6, Temp			; store it
		pop		Temp
		ret									; go out
 MAKE_SEND_REG_NEG:					; Calculate 0-NibConv
 		push	Temp2
		push	TempLow3
		clr		Temp2						; use for subtraction
		mov		TempLow3, NibConv1
		sub		Temp2, TempLow3
		mov		Temp, Temp2					; make a copy
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib1, Temp			; store it
		swap	Temp2
		andi	Temp2, 0b00001111			; only use higher nibble
		sts		ToSendNib2, Temp2			; store it
		clr		Temp2						; use for subtraction
		mov		TempLow3, NibConv2
		sbc		Temp2, TempLow3
		mov		Temp, Temp2					; make a copy
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib3, Temp			; store it
		swap	Temp2
		andi	Temp2, 0b00001111			; only use higher nibble
		sts		ToSendNib4, Temp2			; store it
		clr		Temp2						; use for subtraction
		mov		TempLow3, NibConv3
		sbc		Temp2, TempLow3
		mov		Temp, Temp2					; make a copy
		andi	Temp, 0b00001111			; only use lower nibble
		sts		ToSendNib5, Temp			; store it
		swap	Temp2
		andi	Temp2, 0b00001111			; only use higher nibble
		sts		ToSendNib6, Temp2			; store it
		pop		TempLow3
		pop		Temp2
		pop		Temp
		ret

; ****************************** CALIPER CONTROL ROUTINES ********************************

CALIPER_FAST:							; enter the fast transfer mode
		rcall	CALIPER_STATE					; send state signal
		rcall	DELAY1s
		rcall	CALIPER_ZERO					; send the zero signal
		ret

CALIPER_ZERO:
		sbi		PORTD, 4						; set zero pin high
		rcall	DELAY1s
		cbi		PORTD, 4						; reset zero pin
		ret

CALIPER_STATE:
		sbi		PORTD, 5						; set state pin high
		rcall	DELAY1s							; wait
		cbi		PORTD, 5						; reset state pin
		ret

; *********************************** DELAY ROUTINES *************************************

DELAY10s:
		push	DelayTemp
		ldi		DelayTemp, 10
 DELAYCOUNT10s:
 		rcall	DELAY1s
 		dec		DelayTemp
		brne	DELAYCOUNT10s
		pop		DelayTemp
		ret

DELAY1s:
		push	DelayTemp
		ldi		DelayTemp, 40;20
 DELAYCOUNT1s:
 		rcall	DELAY25ms
 		dec		DelayTemp
		brne	DELAYCOUNT1s
		pop		DelayTemp
		ret

DELAY25ms:
		push	DelayTemp
		ldi		DelayTemp, 255
 DELAYCOUNT25ms:
 		rcall	DELAY775
 		dec		DelayTemp
		brne	DELAYCOUNT25ms
		pop		DelayTemp
		ret

DELAY775:								; *** provide 775 MZ delay including the rcall command
		push	DelayTemp						; 2MZ
		ldi		DelayTemp, 255					; 1MZ
 DELAYCOUNT:
 		dec		DelayTemp						; 1MZ
		brne	DELAYCOUNT						; 1/2MZ
		pop		DelayTemp						; 2MZ
		ret										; 3MZ, rcall 3MZ