.include "m8def.inc"

;*****************************************************************************************
;	Variables
;*****************************************************************************************

.def	OldKey			= r0			; Old Keyboard Activity (DeJitter)
.def	KeyValue		= r1			; Value of pressed key pad, (0x0..0xF)
.def	SendMe			= r2			; Register to send via UART
.def	TimCount		= r3			; For the timer jobs
.def	ConvRes1		= r4			; to store the axis conversion result before digit conversion
.def	ConvRes2		= r5
.def	ConvRes3		= r6
.def	NTemp1			= r7			; e.g. used by HEX_DEZ_CONV
.def	NTemp2			= r8
.def	NTemp3			= r9
.def	CntScaler		= r10			; for the display actions
.def	Temp			= r16			; Temporary Variable
.def	Temp2			= r17			; Temporary Variable
.def	DelayTemp 		= r18
.def	Status1			= r19			; Program State Register
									; Bit0		1=Timer event occured
									; Bits123 	001 Axis1
									;			010 Axis2
									;			011 Axis3
									;			100 Axis4
									; Bit4		1=receive from caliper
									; Bit5		1= value/chain menu active
									; Bit6		1= key pressed, run menu routine
									; Bit7		1= menu routine action after key pressed finished
.def	MenuStatus		= r20			; Status Register for Menu routine
									; Bit0		1=Menu active
									; Bits21	00 Axis1
									;			01 Axis2
									;			10 Axis3
									;			11 Axis4
									; Bit3		1= set direction
									; Bit4		1= set other axis
									; Bit5	    1= set other axis direction
									; Bit6		1= set offset
									; Bit7		1= clear all axes
.def	ReadByte		= r21			; Number of received Byte
										; and axis number during lcd send routine
.def	EditValue		= r22			; Number of edit digit in set value routine
.def	DispNum			= r23		; to display different descriptions

;*****************************************************************************************
;	SRAM-Variables
;*****************************************************************************************

.equ	ConvRes1		= 4				; to address the ConvRes registers
.equ	ConvRes2		= 5
.equ	ConvRes3		= 6

.equ	Ax1Byte1		= 0x60			; Axis 1 Read Value
.equ	Ax1Byte2		= 0x61
.equ	Ax1Byte3		= 0x62

.equ	Ax2Byte1		= 0x64			; Axis 2 Read Value
.equ	Ax2Byte2		= 0x65
.equ	Ax2Byte3		= 0x66

.equ	Ax3Byte1		= 0x68			; Axis 3 Read Value
.equ	Ax3Byte2		= 0x69
.equ	Ax3Byte3		= 0x6A

.equ	Ax4Byte1		= 0x6C			; Axis 4 Read Value
.equ	Ax4Byte2		= 0x6D
.equ	Ax4Byte3		= 0x6E

.equ	Ax1Offs1		= 0x6F			; Axis 1 offset value
.equ	Ax1Offs2		= 0x70
.equ	Ax1Offs3		= 0x71

.equ	Ax2Offs1		= 0x73			; Axis 2 offset value
.equ	Ax2Offs2		= 0x74
.equ	Ax2Offs3		= 0x75

.equ	Ax3Offs1		= 0x77			; Axis 3 offset value
.equ	Ax3Offs2		= 0x78
.equ	Ax3Offs3		= 0x79

.equ	Ax4Offs1		= 0x7B			; Axis 4 offset value
.equ	Ax4Offs2		= 0x7C
.equ	Ax4Offs3		= 0x7D

.equ	Axis1Lcd0		= 0x7E			; Axis 1 value to send to display
.equ	Axis1Lcd1		= 0x7F
.equ	Axis1Lcd2		= 0x80
.equ	Axis1Lcd3		= 0x81
.equ	Axis1Lcd4		= 0x82
.equ	Axis1Lcd5		= 0x83
.equ	Axis1Lcd6		= 0x84
.equ	Axis1Lcd7		= 0x85

.equ	Axis2Lcd0		= 0x86			; Axis 2 value to send to display
.equ	Axis2Lcd1		= 0x87
.equ	Axis2Lcd2		= 0x88
.equ	Axis2Lcd3		= 0x89
.equ	Axis2Lcd4		= 0x8A
.equ	Axis2Lcd5		= 0x8B
.equ	Axis2Lcd6		= 0x8C
.equ	Axis2Lcd7		= 0x8D

.equ	Axis3Lcd0		= 0x8E			; Axis 3 value to send to display
.equ	Axis3Lcd1		= 0x8F
.equ	Axis3Lcd2		= 0x90
.equ	Axis3Lcd3		= 0x91
.equ	Axis3Lcd4		= 0x92
.equ	Axis3Lcd5		= 0x93
.equ	Axis3Lcd6		= 0x94
.equ	Axis3Lcd7		= 0x95

.equ	Axis4Lcd0		= 0x96			; Axis 4 value to send to display
.equ	Axis4Lcd1		= 0x97
.equ	Axis4Lcd2		= 0x98
.equ	Axis4Lcd3		= 0x99
.equ	Axis4Lcd4		= 0x9A
.equ	Axis4Lcd5		= 0x9B
.equ	Axis4Lcd6		= 0x9C
.equ	Axis4Lcd7		= 0x9D

.equ	LcdSave0		= 0x9E			; for the axis recalculation routine
.equ	LcdSave1		= 0x9F
.equ	LcdSave2		= 0xA0
.equ	LcdSave3		= 0xA1
.equ	LcdSave4		= 0xA2
.equ	LcdSave5		= 0xA3
.equ	LcdSave6		= 0xA4
.equ	LcdSave7		= 0xA5

.equ	Axis1State		= 0xA6			; Bit	0  primary axis direction
										;		0  positive
										;		1  negative
										; Bit	1  secondary axis direction
										;		0  positive
										;		1  negative
										; Bits	32 2nd axis
										;		00 1
										;		01 2
										;		10 3
										;		11 4
										; Bit	4  use 2nd axis
										;		0  no
										;		1  yes
										; Bit	5  axis error
										;		0  no error
										;		1  error
.equ	Axis2State		= 0xA7			; axis state registers
.equ	Axis3State		= 0xA8
.equ	Axis4State		= 0xA9

.equ	AxisDesc0		= 0xAA			; axis description
.equ	AxisDesc1		= 0xAB
.equ	AxisDesc2		= 0xAC
.equ	AxisDesc3		= 0xAD
.equ	AxisDesc4		= 0xAE
.equ	AxisDesc5		= 0xAF
.equ	AxisDesc6		= 0xB0
.equ	AxisDesc7		= 0xB1

;*****************************************************************************************
;	EEPROM Variables
;*****************************************************************************************
;.eseg

;		.dw	0x0000			; Drehzahl  2
;		.dw	0x0001			; Drehzahl  3

;*****************************************************************************************
;	Values
;*****************************************************************************************

.equ	LCD_PORT     	= PORTC			; display data connection
.equ	LCD_DDR	     	= DDRC
.equ	LCD_PIN	     	= PINC
.equ	LCD_CRTL     	= PORTC			; display control pin connection
.equ	LCD_CRTL_DDR 	= DDRC
.equ	LCD_CRTL_PIN 	= PINC
.equ	LCD_CRTL_E	 	= 4				; display control pins !!!! switched!!!
.equ	LCD_CRTL_RS	 	= 5
.equ	Trans8Bit		= 0b00000011	; 8 Bit transfer mode command, all commands need 50us
.equ	Trans4Bit		= 0b00000010	; 4 bit transfer mode command
.equ	ClearDisp		= 0b00000001	; clear display command, needs 2ms!
;.equ	CursHome		= 0b00000010	; Send Cursor Home Command, needs 2ms
						;   +--------- Set DD Adress Command
.equ	Start1Line		= 0b10000000
.equ	Start2Line		= 0b11000000	; 40h is Beginning of second line
.equ	Start3Line		= 0b10010000
.equ	Start4Line		= 0b11010000
.equ	Mid1Line		= 0b10001000
;.equ	Mid2Line		= 0b11001000
;.equ	Mid3Line		= 0b10011000
;.equ	Mid4Line		= 0b11011000
			;	Set the Display
				;              +------ Data:  0 = 4-bit; 1 = 8-bit
				;              |+----- Lines: 0 = 1; 1 = 2
				;              ||+---- Font:  0 = 5x8; 1 = 5x11
.equ	SetDisp			= 0b00101100
		; ***  'CURSOR/DISPLAY SHIFT' command
				;               +----- S/C:  0 = cursor; 1 = display
				;		        |+---- R/L:  0 = left; 1 = right
.equ	ShCursRight		= 0b00010100
		; ***  'DISPLAY ON/OFF' command
				;                +---- Display: 0 = off; 1 = on
				;                |+--- Cursor: 0 = off; 1 = on
				;                ||+-- Blink: 0 = off; 1 = on
.equ	DispOn			= 0b00001100
;.equ	CursSmallBlink	= 0b00001101
.equ	CursLargeBlink	= 0b00001111
;.equ	CursSmallOn		= 0b00001110
		; *** 'ENTRY MODE' command
			;					  +--- Direction: 0 = left; 1 = right
			;					  |+-- Shift Dislay: 0 = off; 1 = on
.equ	ShiftCursRight	= 0b00000110

.equ	KEY_PORT		= PORTB			; 16 key keypad connection
.equ	KEY_DDR			= DDRB
.equ	KEY_PIN			= PINB

;.equ	Axis1Adress		= 0x81			; adresses of the 4 axes units
;.equ	Axis2Adress		= 0x82
;.equ	Axis3Adress		= 0x83
;.equ	Axis4Adress		= 0x84

;*****************************************************************************************
;	Program Code
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interrupt Routines
;*****************************************************************************************

		rjmp	START					; Reset Vector
		reti							; Int0 Pin
		reti							; Int1 Pin
		reti							; Timer 2 Compare
		reti							; Timer 2 Overflow
		reti							; Timer 1 Capture
		reti							; Timer 1 Compare A
		reti							; Timer 1 Compare B
		reti							; Timer 1 Overflow
		rjmp	T0OVFROUTINE			; Timer 0 Overflow
		reti							; SPI Transfer Complete
		rjmp	RXCROUTINE				; UART RX Complete
		rjmp	UDREROUTINE				; UART UDRE Routine
		reti							; UART TX Complete
		reti							; ADC Conversion complete
		reti							; EEPROM ready
		reti							; Analog comparator
		reti							; two wire interface
		reti							; Store program memory ready

;*****************************************************************************************
;	Data for the Display
;*****************************************************************************************
Clear16:			.db "                "
AxErr:				.db "  Axis   Error  "
Welcome:			.db " Elektronisches "
Welc2:				.db " 4-Kanal"
Welc3:				.db	" Messystem      "

Main1stLinea:		.db "A..Primary Axis "
Main2ndLinea:		.db "  Direction     "
Main1stLineb:		.db "B Secondary Axis"
Main2ndLineb:		.db "     0...4      "
Main1stLinec:		.db "C Secondary Axis"
Main1stLined:		.db "D ... Set New   "
Main2ndLined:		.db " Display Value  "
Main1stLinee:		.db	"0..Set Axis to 0"

PrimSign1stLinea:	.db "  Primary Axis  "
PrimSign2ndLinea:	.db "   Direction    "
PrimSign1stLineb:	.db "A..Positive Sign"
PrimSign2ndLineb:	.db "B..Negative Sign"

SecAxChoose1stLine:	.db " Secondary Axis "
SecAxChoose2ndLine:	.db "      0..4      "

SecSign1stLinea:	.db	" Secondary Axis "

SetVal1stLinea:		.db	"Set new Display "
SetVal2ndLinea:		.db " Value  0...9   "
SetVal1stLineb:		.db	"A Positive/Back "
SetVal2ndLineb:		.db	"B Negative/Forw."

DecChainVal1stLine:	.db "A Set as new Val"
DecChainVal2ndLine:	.db	"B Chain Addition"

Main3rdLine: 		.db "  E/F .. Escape "

;*****************************************************************************************
;	Program Start
;*****************************************************************************************
START:
		ldi		Temp, low(RAMEND)				; init stack pointer
		out		SPL, Temp
		ldi		Temp, high(RAMEND)
		out		SPH, Temp
		ldi		Temp, 5
		mov		TimCount, Temp					; set the TimCount register
		ldi		Temp, 80
		mov		CntScaler, Temp					; 3s change rate
		clr		Status1							; clear program state
		out		GICR, Status1					; Disable Interrupts Int0 and Int1
		out		TIMSK, Status1					; Disable all Timer Interrupts
		mov		KeyValue, Status1
		mov		MenuStatus, Status1				; clear menu state
		mov		EditValue, Status1
		mov		ReadByte, Status1
		sts		Axis1State, Status1				; reset all Axis states
		sts		Axis2State, Status1
		sts		Axis3State, Status1
		sts		Axis4State, Status1
		sts		Ax1Offs1, Status1				; clear all offset registers
		sts		Ax1Offs2, Status1
		sts		Ax1Offs3, Status1
		sts		Ax2Offs1, Status1
		sts		Ax2Offs2, Status1
		sts		Ax2Offs3, Status1
		sts		Ax3Offs1, Status1
		sts		Ax3Offs2, Status1
		sts		Ax3Offs3, Status1
		sts		Ax4Offs1, Status1
		sts		Ax4Offs2, Status1
		sts		Ax4Offs3, Status1
		ldi		Temp, 0XFF
		mov		OldKey, Temp					; no key pressed
		ldi		Temp, 0b11110000				; set upper 4 keypad pins to output
		out		KEY_DDR, Temp
		ldi		Temp, 0b00001111				; set display data pins to output
		out		KEY_PORT, Temp					; pull them low and the others weak high
		out		LCD_DDR, Temp
		sbi		LCD_CRTL_DDR, LCD_CRTL_E		; set display control pins
		sbi		LCD_CRTL_DDR, LCD_CRTL_RS
		clr		Temp
		out		UCSRA, Temp						; disable mcpm and u2x
		out		UBRRH, Temp
		ldi		Temp, 18						; set Baud rate to 19200@4MHz
		out		UBRRL, Temp
		ldi		Temp, 0b00011000				; enable RX and TX Pin
		out		UCSRB, Temp						; RXCIE, TXCIE, UDRIE, RXEN, TXEN, UCSZ2, RXB8, TXB8
		ldi		Temp, 0b10001110
		out		UCSRC, Temp						; URSEL, UMSEL, UPM1, UPM0, USBS, UCSZ1, UCSZ0, UCPOL
		rcall	DELAY1s							; time for the Display to start up
		rcall	DISP_TRANS						; set the Transfer mode
		ldi		Temp, ClearDisp					; clear the Display
		rcall	LCD_SEND_CMD					; wait after clearing
		rcall	DELAY4ms
		ldi		Temp, SetDisp					; Set Up the Display Lines and Fonts
		rcall	LCD_SEND_CMD
		ldi		Temp, ShCursRight				; Set Display/Cursor Shift and Direction
		rcall	LCD_SEND_CMD
		ldi		Temp, DispOn					; Set Display/Cursor ON/OFF
		rcall	LCD_SEND_CMD
		ldi		Temp, ShiftCursRight			; Set the Cursor/Display Right/left Shift during Entry
		rcall	LCD_SEND_CMD
		ldi		Temp, Start1Line				; Send Cursor to Beginning of first line
		rcall	LCD_SEND_CMD					; wait after sending cursor home
		ldi		ZL, low(Welcome*2)				; Set Pointer to Menu Text
		ldi		ZH, high(Welcome*2)
		ldi		Temp2, 16
		rcall	LCD_PRINT_PM					; Print the Text
		ldi		Temp, Start2Line				; Send Cursor to beginning of second Line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM					; Print the next Line
		ldi		Temp, Start3Line				; Send Cursor to beginning of third Line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16
		rcall	LCD_PRINT_PM					; Print the Text
		sei										; enable interrupts
		rcall	DELAY10s
		ldi		Temp, 0x91
		mov		SendMe, Temp					; set axis 1 zero
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 2 zero	
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 3 zero	
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 4 zero	
		sbi		UCSRB, UDRIE
		rcall	DELAY10s
		in		Temp, UDR
		ldi		Temp, 0xB1
		mov		SendMe, Temp					; set axis 1 fast
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 2 fast
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 3 fast
		sbi		UCSRB, UDRIE
		rcall	DELAY4ms
		inc		SendMe							; set axis 4 fast
		sbi		UCSRB, UDRIE
		ldi		Temp, ClearDisp					; Clear the Display
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		Temp, 0b00000001				; enable T0OVF interrupt
		out		TIMSK, Temp						; OCIE2, TOIE2, TICIE1, OCIE1A, OCIE1B, TOIE1, ,T0IEO,
		ldi		Temp, 0b00000101				; set clock prescaler to 1024 (4kHz timer clock)
		out		TCCR0, Temp						;  , , , , , CS2, CS1, CS0
		ldi		Temp, 0b00101100				; load ","
		sts		Axis1Lcd5, Temp					; store it to the ","-places
		sts		Axis2Lcd5, Temp
		sts		Axis3Lcd5, Temp
		sts		Axis4Lcd5, Temp

;********************************* MAIN ROUTINE ******************************************		

MAIN:
		sbrc	Status1, 0
		rjmp	MAIN_TIMER_EVENT				; start timer event routine after event occured
		rjmp	MAIN

MAIN_TIMER_EVENT:
		andi	Status1, 0b11111110				; reset timer event state
		clr		Temp							; reset verify register
		cp		TimCount, Temp
		brne	MAIN_TIMER_NO_0
		rcall	KEYBOARD_READ					; read the keyboard 2 times every loop
 		rjmp	MAIN_RUN_AX						; first read axis 1
 MAIN_TIMER_NO_0:
		ldi		Temp, 4
		cp		TimCount, Temp
		brlo	MAIN_RUN_AX
		brne	MAIN_TIMER_NO_4
		rjmp	MAIN_RUN_AX_CONV				; convert all axes
 MAIN_TIMER_NO_4:
		rcall	MAIN_TRANS_LCD					; output to display
		rjmp	MAIN

 MAIN_TO_KEY_READ:
		rcall	KEYBOARD_READ					; read the keyboard
		sbrc	Status1, 6						; only run when key changed
		rcall	KEYBOARD_ACTION
		rjmp	MAIN

;*****************************************************************************************
;	Interrupt Routines
;*****************************************************************************************

T0OVFROUTINE:							; manage the TimCount value to start different routines
		push	Temp
		mov		Temp, TimCount					; load the counter register
		inc		Temp							; increase it
		cpi		Temp, 6
		brlo	T0OVFROUTINE_NORMAL
		clr		Temp							; reset timer after 5 reached
 T0OVFROUTINE_NORMAL:
		mov		TimCount, Temp					; save it
		ori		Status1, 0b00000001				; set timer event state
		ldi		Temp, 230
		out		TCNT0, Temp						; set timer to 6ms
		pop		Temp
		reti

RXCROUTINE:								; store the received nibble to Low(Y) (0,2,4)
												; to high(Y) (1,3,5) , increase Y (1,3,5)
												; increase ReadByte every cycle
												; stop after 6 reads
		push	Temp
		push	Temp2
		clr		Temp2
		in		Temp2, UCSRA
		in		Temp, UDR						; read data
		andi	Temp2, (1<<FE)|(1<<DOR)
		brne	RXCROUTINE_OUT					; go out if error occured 
		sbrs	Status1, 4
		rjmp	RXCROUTINE_OUT					; skip if failure transmission
		sbrs	ReadByte, 0
		rjmp	RXCROUTINE_EVEN					; read the lower nibble, 0,2,4
		ld		Temp2, Y						; load the lower nibble
		swap	Temp							; swap low(Temp) to high(Temp)
		or		Temp, Temp2						; put lower and higher nibble together
 RXCROUTINE_EVEN:
		st		Y, Temp							; store it
		sbrc	ReadByte, 0
		inc		YL								; increase Y after higher nibble stored (1,3,5)
		inc		ReadByte						; increase Nr. of received Bytes
		cpi		ReadByte, 6						; 6 nibbles are to receive
		breq	RXCROUTINE_STATE_RESET
 RXCROUTINE_OUT:
 		pop		Temp2
		pop		Temp
		reti
 RXCROUTINE_STATE_RESET:				; reset Status1 after receive complete
		cbi		UCSRB, RXCIE					; disable receive interrupt
		andi	Status1, 0b11100001				; reset the receive states
 		rjmp	RXCROUTINE_OUT

UDREROUTINE:
		out		UDR, SendMe						; output data
		cbi		UCSRB, UDRIE					; disable UDRE interrupt
		reti

;*****************************************************************************************
;	Subroutines
;*****************************************************************************************

MAIN_RUN_AX:							; init the caliper transfer from the caliper units
		push	Temp
		mov		Temp, Timcount					; test if 1st caliper transfer
		cpi		Temp, 1
		brlo	MAIN_RUN_AX_OK					; 1st caliper transfer, skip error test
		sbrs	Status1, 4						; test if transfer finished
		rjmp	MAIN_RUN_AX_OK					; transfer finished, go on with send routine
		ldi		XL, LOW(Axis1State)				; point to axis 1 state register
		ldi		XH, HIGH(Axis1State)
		dec		Temp							; get previous axis counter
		add		XL, Temp						; use correct state register
		ld		Temp, X							; load the state register
		ori		Temp, 0b00100000				; set the error state
		st		X, Temp							; store the modified state register
  MAIN_RUN_AX_OK:
		sbi		UCSRB, RXCIE					; enable RXC interrupt
		ldi		Temp, 0x81						; load 1st caliper adress
		mov		SendMe, Temp					; move it to the send register
		add		SendMe, TimCount				; adress the correct caliper
		ldi		Temp, 0b00001001				; load axis 1 read state
		add		Temp, TimCount					; add the correct axis
		lsl		Temp							; move to correct position
		andi	Status1, 0b11100000
		or		Status1, Temp					; set axis  read state
		ldi		YL, LOW(Ax1Byte1)				; set adress pointer to axis1
		ldi		YH, HIGH(Ax1Byte1)				; set adress pointer to axis1
		mov		Temp, TimCount					; load axis counter
		lsl		Temp							; multiply by 4
		lsl		Temp
		add		YL, Temp						; use the correct axis
		clr		ReadByte						; set Nr. of received Bytes zero
		sbi		UCSRB, UDRIE					; enable UDRE interrupt to send request
		pop		Temp
		rjmp	MAIN

 MAIN_RUN_AX_CONV:						; do inversion (when negative), other axis and offset calculation
												; convert into lcd value
		push	Temp
		push	Temp2
		cli										; disable interrupts
		sbrs	Status1, 4						; axis 4 ok when Status1, 4 =0
		breq	MAIN_RUN_AX_CONV_4_OK
		lds		Temp, Axis4State
		ori		Temp, 0b00100000				; set axis 4 error
		sts		Axis4State, Temp
  MAIN_RUN_AX_CONV_4_OK:						; calculate axis 1 value
		clr		Temp2
   MAIN_RUN_AX_CONV_INNER:
		ldi		XH, HIGH(Axis1State)
		ldi		XL, LOW(Axis1State)
		add		XL, Temp2						; adress the correct State register
		ld		Temp, X							; load axis state
		sbrc	Temp, 5
		rjmp	MAIN_RUN_AX_CONV_NEXT			; error, go to next axis
		ldi		YL, LOW(Ax1Offs1)				; load address of axis 1 offset
		ldi		YH, HIGH(Ax1Offs1)
		mov		Temp, Temp2						; backup of axis count
		lsl		Temp							; multiply by 4
		lsl		Temp
		add		YL, Temp						; adress the correct axis offset
		ldi		ZL, LOW(Ax1Byte1)				; load address of axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		add		ZL, Temp						; adress the correct axis
		ld		Temp, X							; load axis state again
		sbrc	Temp, 0
		rcall	SUB_2_VAL						; subtract when bit set
		sbrs	Temp, 0
		rcall	ADD_2_VAL						; add when not set
		sbrs	Temp, 4							; use 2nd axis when set
		rjmp	MAIN_RUN_AX_CONV_NO_2ND
		ldi		YL, LOW(ConvRes1)
		ldi		YH, HIGH(ConvRes1)				; load result and 2nd axis to add/subtract
		andi	Temp, 0b00001100				; only use the 2nd axes bits
		ldi		ZL, LOW(Ax1Byte1)				; load address of axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		add		ZL, Temp						; adress the correct 2nd axis
		ld		Temp, X							; load axis state again
		sbrc	Temp, 1							; subtract when set
		rcall	SUB_2_VAL
		sbrs	Temp, 1							; add when clear
		rcall	ADD_2_VAL
  MAIN_RUN_AX_CONV_NO_2ND:						; convert ConvRes1..3 to the lcd value and
		ldi		ZL, LOW(Axis1Lcd0)				; store it to AxisxxLcd
		ldi		ZH, HIGH(Axis1Lcd0)				; load Axis1Lcd
		mov		Temp, Temp2						; use loop count
		lsl		Temp							; multiply by 8
		lsl		Temp
		lsl		Temp
		add		ZL, Temp						; adress the correct lcd register
		rcall	HEX_DEZ_CONV					; convert to lcd value
  MAIN_RUN_AX_CONV_NEXT:						; go to next axis or go out
		inc		Temp2							; increase loop count
		cpi		Temp2, 4
		brlo	MAIN_RUN_AX_CONV_INNER			; run until all axes finished
		sei										; enable interrupts
		pop		Temp2							; go out
		pop		Temp
		rjmp	MAIN_TO_KEY_READ

KEYBOARD_ACTION:						; navigate through the menu states depending
												; on the key pressed, change MenuStatus and
												; AxisState and
												; recalculate values depending on state
												; and input
		sbrc	Status1, 7						; check if routine already runned
		ret										; go back if no new key action
		push	Temp
		push	Temp2
		cli										; disable interrupts
		mov		Temp2, KeyValue					; load the key value
		sbrs	MenuStatus, 0					; test if menu already active
		rjmp	KEYBOARD_ACTION_START_MENU_TEST	; go into menu start routine
		cpi		Temp2, 0x0F						; end menu with F key
		brne	KEYBOARD_ACTION_GO_ON
		clr		MenuStatus						; end the state and go out
		rjmp	KEYBOARD_ACTION_GO_OUT
 KEYBOARD_ACTION_GO_ON:
		cpi		MenuStatus, 0b00001001
		brlo	KEY_ACT_AXES_MAIN				; all axes main routine
		sbrc	MenuStatus, 3
		rjmp	KEY_ACT_PRIM_SIGN				; change the sign of actual axis
		sbrc	MenuStatus, 4
		rjmp	KEY_ACT_2ND_AXIS				; set 2nd axis
		sbrc	MenuStatus, 5
		rjmp	KEY_ACT_2ND_SIGN				; set 2nd axis sign
		sbrc	MenuStatus, 6
		rjmp	KEY_ACT_VAL						; set display value
		clr		MenuStatus						; to prevent hang-ups
		rjmp	KEYBOARD_ACTION_GO_OUT
 KEYBOARD_ACTION_START_MENU_TEST:				; start the menu routine for ax 1
												;  to 4 depending on pressed key
 		cpi		Temp2, 5
		brsh	KEYBOARD_ACTION_GO_OUT_JMP		; go out if no axis key pressed
		cpi		Temp2, 1
		brlo	KEY_SET_ALL_0					; set all axes to zero
		dec		Temp2							; to adress the correct axis
		lsl		Temp2
		ori		Temp2, 0b00000001				; add menu active bit
		mov		MenuStatus, Temp2				; set it to new menu state
		rjmp	KEYBOARD_ACTION_GO_OUT

		;********************************* JUMP ROUTINES *********************************	
KEYBOARD_ACTION_GO_OUT_JMP:
 		rjmp	KEYBOARD_ACTION_GO_OUT	
		;*********************************************************************************
 KEY_SET_ALL_0:
		clr		Temp2							; clear value counter
  KEY_SET_ALL_0_INNER:
		clr		Temp
		mov		ConvRes1, Temp					; clear the new axis value registers
		mov		ConvRes2, Temp
		mov		ConvRes3, Temp
		mov		Temp, Temp2
		lsl		Temp
		ori		Temp, 0b10000000				; set the menu state for the axes
		mov		MenuStatus, Temp
		sbrc	Temp2, 3						; stop if axis 4 reached
		rjmp	KEY_SET_ALL_0_STOP
		inc		Temp2							; increase value counter
		rjmp	KEY_ACT_VAL_REC_ABS				; calculate new offset
  KEY_SET_ALL_0_STOP:
		clr		MenuStatus						; reset menu state
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out
 KEY_ACT_AXES_MAIN:							; main menu routine for axes 1..4
 												; set the MenuStatus depending on
												; the key pressed
		cpi		Temp2, 0x00
		brne	KEY_ACT_AXES_MAIN_NO_0			; set zero key pressed
		clr		Temp							; clear the new value register
		mov		ConvRes1, Temp
		mov		ConvRes2, Temp
		mov		ConvRes3, Temp
		rjmp	KEY_ACT_VAL_REC_ABS				; calculate new axis offset
  KEY_ACT_AXES_MAIN_NO_0:
		cpi		Temp2, 0x01						; set caliper zero key pressed
		brne	KEY_ACT_AXES_MAIN_NO_1
		ldi		Temp2, 0x90						; load set zero command
		rjmp	KEY_ACT_AXES_MAIN_SEND_COMMAND	; send the command
  KEY_ACT_AXES_MAIN_NO_1:
		cpi		Temp2, 0x02						; set caliper state key pressed
		brne	KEY_ACT_AXES_MAIN_NO_2
		ldi		Temp2, 0xA0						; load set state command
		rjmp	KEY_ACT_AXES_MAIN_SEND_COMMAND	; send the command
  KEY_ACT_AXES_MAIN_NO_2:
		cpi		Temp2, 0x03						; set caliper fast transfer key pressed
		brne	KEY_ACT_AXES_MAIN_NO_3
		ldi		Temp2, 0xB0						; load set fast transfer mode command
   KEY_ACT_AXES_MAIN_SEND_COMMAND:
		clr		Temp
		out		TCCR0, Temp						; stop the timer
		mov		Temp, Menustatus
		andi	Temp, 0b00000110				; extract current axis to Temp
		lsr		Temp
		inc		Temp
		or		Temp, Temp2
		mov		SendMe, Temp
		sbi		UCSRB, UDRIE					; send the command
		rcall	DELAY1s							; wait
		rcall	DELAY1s
		rcall	DELAY1s
		rcall	DELAY1s
 		ldi		Temp, 0b00000101				; set clock prescaler to 1024 (4kHz timer clock)
		out		TCCR0, Temp						; restart the timer
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out	
  KEY_ACT_AXES_MAIN_NO_3:
		cpi		Temp2, 0x0A
		brne	KEY_ACT_AXES_MAIN_NO_A			; try next if not A pressed
		ori		MenuStatus, 0b00001000			; routine set primary direction
		rjmp	KEYBOARD_ACTION_GO_OUT
  KEY_ACT_AXES_MAIN_NO_A:
 		cpi		Temp2, 0x0B
		brne	KEY_ACT_AXES_MAIN_NO_B			; try next if not B pressed
		ori		MenuStatus, 0b00010000			; routine set other axis
		rjmp	KEYBOARD_ACTION_GO_OUT
  KEY_ACT_AXES_MAIN_NO_B:
		cpi		Temp2, 0x0C						; try next if not C pressed
		brne	KEY_ACT_AXES_MAIN_NO_C
		ori		MenuStatus, 0b00100000			; routine set other axis sign
		rjmp	KEYBOARD_ACTION_GO_OUT
  KEY_ACT_AXES_MAIN_NO_C:
  		cpi		Temp2, 0x0D						; try next if not D pressed
		brne	KEY_ACT_AXES_MAIN_NO_D
		ori		MenuStatus, 0b01000000			; routine set offset
		ldi		ZH, HIGH(Axis1Lcd0)
		ldi		ZL, LOW(Axis1Lcd0)				; adress axis 1 lcd register
		mov		Temp, MenuStatus				; use axes bits of MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 8
		lsl		Temp
		add		ZL, Temp						; adress the correct lcd register
		clr		Temp
		mov		EditValue, Temp					; clear EditValue
												; backup display value and go on
		ld		Temp, Z+						; load sign
		sts		LcdSave0, Temp
		ld		Temp, Z+						; load 100000er value
		sts		LcdSave1, Temp
		ld		Temp, Z+						; load 10000er value
		sts		LcdSave2, Temp
		ld		Temp, Z+						; load 1000er value
		sts		LcdSave3, Temp
		ld		Temp, Z+						; load 100er value
		sts		LcdSave4, Temp
		ld		Temp, Z+						; load komma sign
		sts		LcdSave5, Temp
		ld		Temp, Z+						; load 10er value
		sts		LcdSave6, Temp
		ld		Temp, Z							; load 1er value
		sts		LcdSave7, Temp
		rjmp	KEYBOARD_ACTION_GO_OUT
  KEY_ACT_AXES_MAIN_NO_D:						; go out
		cpi		Temp2, 0x0E						; E is escape key
		brne	KEY_ACT_AXES_MAIN_NO_E
		clr		MenuStatus						; go out from menu routine
  KEY_ACT_AXES_MAIN_NO_E:
		rjmp	KEYBOARD_ACTION_GO_OUT
 KEY_ACT_PRIM_SIGN:								; change the sign of actual axis
												; load Readaxis into ConvRes1..3
		ldi		XL, LOW(Axis1State)
		ldi		XH, HIGH(Axis1State)			; load Axis1State adress
		ldi		YL, LOW(Ax1Offs1)				; address axis 1 offset
		ldi		YH, HIGH(Ax1Offs1)
		ldi		ZL, LOW(Ax1Byte1)				; address axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		mov		Temp, MenuStatus				; use axis bits of MenuStatus again
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 4
		add		YL, Temp						; adress the correct offset
		add		ZL, Temp						; adress the correct axis
		lsr		Temp							; use original axis value
		lsr		Temp
		add		XL, Temp						; use correct AxisState register adress
		ld		Temp, X							; load the AxisState register
												; load the addressed axis to ConvRes and go on
		ld		ConvRes1, Z+ 					; load the axis bytes
		ld		ConvRes2, Z+
		ld		ConvRes3, Z
		lsl		ConvRes1						; multiply ConvRes by 2
		rol		ConvRes2
		rol		ConvRes3
		cpi		Temp2, 0x0A
		breq	KEY_ACT_PRIM_POS				; A is positive sign
		cpi		Temp2, 0x0B
		breq	KEY_ACT_PRIM_NEG				; B is negative sign
		cpi		Temp2, 0x0E
		breq	KEY_ACT_SUB_ESCAPE				; E is escape
		rjmp	KEYBOARD_ACTION_GO_OUT			; just go out if other key pressed
  KEY_ACT_PRIM_POS:
		sbrs	Temp, 0							; verify primary direction state
		rjmp	KEY_ACT_SUB_ESCAPE				; go out as same sign
		andi	Temp, 0b11111110				; set positive direction
		st		X, Temp							; store changed register
		ldi		ZL, LOW(ConvRes1)				; 2nd pointer to 2x readaxis
		rcall	SUB_2_VAL_Y						; offset=offset-2*primaxis
		rjmp	KEY_ACT_SUB_ESCAPE				; go one menu up and out
  KEY_ACT_PRIM_NEG:
		sbrc	Temp, 0							; verify primary direction state
		rjmp	KEY_ACT_SUB_ESCAPE				; go out as same sign
		ori		Temp, 0b00000001				; change sign in register to negative
		st		X, Temp							; store changed register
		ldi		ZL, LOW(ConvRes1)				; 2nd pointer to 2x readaxis
		rcall	ADD_2_VAL_Y						; offset=offset+2*primaxis
  KEY_ACT_SUB_ESCAPE:							; go back to axes menu and out
		andi	MenuStatus, 0b00000111			; reset to main menu state
 		rjmp	KEYBOARD_ACTION_GO_OUT

 KEY_ACT_2ND_AXIS:								; set 2nd axis
		cpi		Temp2, 0x0E						; escape button
		breq	KEY_ACT_SUB_ESCAPE				; escape when button pressed
		ldi		XL, LOW(Axis1State)
		ldi		XH, HIGH(Axis1State)			; point to axis 1 state register
		ldi		YL, LOW(Ax1Offs1)				; point to axis 1 offset
		ldi		YH, HIGH(Ax1Offs1)
		ldi		ZL, LOW(Ax1Byte1)				; point to axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		mov		Temp, MenuStatus				; use axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 4
		add		YL, Temp						; use correct axis offset
		lsr		Temp							; use original axis counter
		lsr		Temp
		add		XL, Temp						; use correct state register adress
		ld		Temp, X							; load axis state
		sbrs	Temp, 4							; skip 2nd axis when to ignore
		rjmp	KEY_ACT_2ND_AXIS_NO_2ND
		andi	Temp, 0b00001100				; only use the 2nd axis bits
		add		ZL, Temp						; adress the correct 2nd axis
		ld		Temp, X							; load axis state again
		sbrs	Temp, 1							; skip when negative
		rcall	ADD_2_VAL_Y						; offset=offset+axis2(old)
		sbrc	Temp, 1							; skip when positive
		rcall	SUB_2_VAL_Y						; offset=offset-axis2(old)
		dec		YL								; re-point to original offset
		dec		YL
   KEY_ACT_2ND_AXIS_NO_2ND:
		cpi		Temp2, 0
		brne	KEY_ACT_2ND_SET_NO_0			; not 0 pressed
		andi	Temp, 0b11101111				; set new 2nd axis to ignore
		st		X, Temp
		rjmp	KEY_ACT_SUB_ESCAPE				; update MenuStatus(go back) and go out
   KEY_ACT_2ND_SET_NO_0:
		cpi		Temp2, 5						; test if invalid key pressed
		brsh	KEYBOARD_ACTION_GO_OUT_JMP1		; just ignore false buttons
		andi	Temp, 0b11110011				; clear old axis bits
		dec		Temp2							; convert to axis definition
		lsl		Temp2							; move Temp2 axis bits into AxisState position
		lsl		Temp2
		ldi		ZL, LOW(Ax1Byte1)				; point to axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		add		ZL, Temp2						; point to correct 2nd axis
		or		Temp, Temp2						; put new axis into state
		ori		Temp, 0b00010000				; set new 2nd axis to use
		st		X, Temp							; store modified state register
		sbrc	Temp, 1							; inverse sign as old axis
		rcall	ADD_2_VAL_Y						; offset=offset+axis2
		sbrs	Temp, 1
		rcall	SUB_2_VAL_Y						; offset=offset-axis2
		andi	MenuStatus, 0b11101111			; update MenuStatus(go back)
   KEYBOARD_ACTION_GO_OUT_JMP1:
		rjmp	KEYBOARD_ACTION_GO_OUT

 KEY_ACT_2ND_SIGN:								; set 2nd axis sign
		cpi		Temp2, 0x0E						; escape button
		breq	KEY_ACT_SUB_ESCAPE				; escape from menu
		ldi		XL,	LOW(Axis1State)				; point to axis 1 state
		ldi		XH, HIGH(Axis1State)
		ldi		YL, LOW(Ax1Offs1)				; point to axis 1 offset
		ldi		YH, HIGH(Ax1Offs1)
		ldi		ZL, LOW(Ax1Byte1)				; point to axis 1
		ldi		ZH, HIGH(Ax1Byte1)
		mov		Temp, MenuStatus				; use active Axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 4
		add		YL, Temp						; use correct offset register
		lsr		Temp
		lsr		Temp							; use original axis count
		add		XL, Temp						; adress correct axis state
		ld		Temp, X							; load axis state
		sbrs	Temp, 4							; 
		rjmp	KEY_ACT_SUB_ESCAPE				; escape from menu
		andi	Temp, 0b00001100				; only use the 2nd axis bits
		add		ZL, Temp						; use the correct 2nd axis
		ld		ConvRes1, Z+					; load 2nd axis into ConvRes
		ld		ConvRes2, Z+
		ld		ConvRes3, Z
		lsl		ConvRes1						; multiply ConvRes by 2
		rol		ConvRes2
		rol		ConvRes3
		ld		Temp, X							; load axis state
		ldi		ZL, LOW(ConvRes1)				; set pointer to ConvRes
		cpi		Temp2, 0x0A						; A is positive sign
		breq	KEY_ACT_2ND_SIGN_POS
		cpi		Temp2, 0x0B						; B is negative sign
		breq	KEY_ACT_2ND_SIGN_NEG
		rjmp	KEYBOARD_ACTION_GO_OUT			; just go out if other key pressed
  KEY_ACT_2ND_SIGN_POS:
		sbrs	Temp, 1
		rjmp	KEY_ACT_SUB_ESCAPE				; escape from menu as same sign
		andi	Temp, 0b11111101				; change from negative to positive direction
		st		X, Temp							; store modified axis state
		rcall	SUB_2_VAL_Y						; offset=offset-2*axis..
		rjmp	KEY_ACT_SUB_ESCAPE				; escape from menu
  KEY_ACT_2ND_SIGN_NEG:
		sbrc	Temp, 1
		rjmp	KEY_ACT_SUB_ESCAPE				; escape from menu as same sign
		ori		Temp, 0b00000010				; change from positive to negative direction
		st		X, Temp							; store modified axis state
		rcall	ADD_2_VAL_Y						; offset=offset+2*axis..
		rjmp	KEY_ACT_SUB_ESCAPE				; escape from menu

 KEY_ACT_VAL:									; set the actual display value
		sbrc	Status1, 5						; test if desition byte active
		rjmp	KEY_ACT_VAL_DECIDE				; jump to desition menu
		cpi		Temp2, 0x0E						; escape key
		brne	KEY_ACT_VAL_NO_ESC
		rjmp	KEY_ACT_VAL_REC_OUT				; escape from menu
  KEY_ACT_VAL_NO_ESC:
		ldi		ZL, LOW(LcdSave0)				; restore digit adress
		ldi		ZH, HIGH(LcdSave0)
		add		ZL, EditValue
		cpi		EditValue, 0					; routine just started
		brne	KEY_ACT_VAL_1_7
		cpi		Temp2, 0x0A						; A is positive sign
		brne	KEY_ACT_VAL_SIGN_NO_POS
		ldi		Temp, 0b00100000				; load " " to first digit
		st		Z, Temp							; store sign
		inc		EditValue						; increase EditValue
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out
  KEY_ACT_VAL_SIGN_NO_POS:
		cpi		Temp2, 0x0B						; B is negative sign
		breq	KEY_ACT_VAL_SIGN_NEG
		rjmp	KEYBOARD_ACTION_GO_OUT			; ignore false buttons
  KEY_ACT_VAL_SIGN_NEG:
 		ldi		Temp, 0b00101101				; load "-" to first digit
		st		Z, Temp							; store sign
		inc		EditValue						; increase EditValue
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out
  KEY_ACT_VAL_1_7:								; edit the values
		cpi		Temp2, 0x0A						; go 1 digit back
		brne	KEY_ACT_VAL_1_7_NO_A
		cpi		EditValue, 6
		brne	KEY_ACT_VAL_1_7_A_NO_6
		dec		EditValue						; skip the komma sign
  KEY_ACT_VAL_1_7_A_NO_6:
		dec		EditValue						; decrease counter
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out
  KEY_ACT_VAL_1_7_NO_A:
		cpi		Temp2, 0x0B						; go 1 digit forward
		brne	KEY_ACT_VAL_1_7_NO_B
		cpi		EditValue, 7
		breq	KEY_ACT_VAL_SET_DECIDE			; end the value input
		cpi		EditValue, 4
		brne	KEY_ACT_VAL_1_7_B_NO_4
		inc		EditValue						; skip the komma sign
  KEY_ACT_VAL_1_7_B_NO_4:
		inc		EditValue						; increase counter
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out
  KEY_ACT_VAL_1_7_NO_B:
		cpi		Temp2, 0x0A
		brlo	KEY_ACT_VAL_1_7_NORM_VAL		; go to normal value calculation
		rjmp	KEYBOARD_ACTION_GO_OUT			; ignore false buttons
  KEY_ACT_VAL_1_7_NORM_VAL:
		ldi		Temp, 0x30						; to convert to ascii
		add		Temp, Temp2						; convert digit to ascii
		st		Z, Temp							; save the digit to LcdSave
		inc		EditValue						; increase EditValue
		cpi		EditValue, 5
		brne	KEY_ACT_VAL_1_7_NORM_NO_5
		inc		EditValue						; skip the komma sign
  KEY_ACT_VAL_1_7_NORM_NO_5:
		sbrc	EditValue, 3
  KEY_ACT_VAL_SET_DECIDE:
		ori		Status1, 0b00100000				; set value/chain desition active if last input finished
		rjmp	KEYBOARD_ACTION_GO_OUT			; go out if not last value
  KEY_ACT_VAL_DECIDE:							; decition between value and chain mode
		cpi		Temp2, 0x0A						; normal value edit
		breq	KEY_ACT_VAL_1_7_RECALC			; normal recalculation
		cpi		Temp2, 0x0B						; chain edit
		breq	KEY_ACT_VAL_CHAIN
		cpi		Temp2, 0x0E						; escape key
		breq	KEY_ACT_VAL_REC_OUT_JMP			; escape from menu
		rjmp	KEYBOARD_ACTION_GO_OUT			; ignore all other keys

		;********************************* JUMP ROUTINES *********************************	
KEY_ACT_VAL_REC_OUT_JMP:
 		rjmp	KEY_ACT_VAL_REC_OUT
KEY_ACT_VAL_1_7_RECALC_JMP:
		rjmp	KEY_ACT_VAL_1_7_RECALC
		;*********************************************************************************

  KEY_ACT_VAL_CHAIN:							; chain mode
		andi	Status1, 0b11011111				; reset value/chain desition bit
		rcall	LCD_HEX_CONV					; convert it to hex and store to ConvRes
		lds		Temp, LcdSave0					; load the sign
		cpi		Temp, 0b00101101				; test if negative
		brne	KEY_ACT_VAL_CHAIN_CALC			; positive value
		clr		Temp
		sub		Temp, ConvRes1					; convert negative --> positive
		mov		ConvRes1, Temp
		clr		Temp
		sbc		Temp, ConvRes2
		mov		ConvRes2, Temp
		clr		Temp
		sbc		Temp, ConvRes3
		mov		ConvRes3, Temp
   KEY_ACT_VAL_CHAIN_CALC:
		ldi		XL, LOW(Axis1State)				; point to Axis1State
		ldi		XH, HIGH(Axis1State)
		ldi		YL, LOW(ConvRes1)				; point to ConvRes
		ldi		YH, HIGH(ConvRes1)
		ldi		ZL, LOW(Ax1Offs1)				; point to 1st axis
		ldi		ZH, HIGH(Ax1Offs1)
		mov		Temp, MenuStatus				; use axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsr		Temp							; use original axis count
		add		XL, Temp						; use correct axis state
		lsl		Temp
		lsl		Temp							; multiply by 4
		add		ZL, Temp						; use correct primary axis
		rcall	ADD_2_VAL						; ConvRes1..3=ConvRes1..3+AxOffs
		ldi		YL, LOW(Ax1Offs1)				; point to axis 1 offset
		mov		Temp, MenuStatus				; use axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 4
		add		YL, Temp						; use the correct offset register
		st		Y, ConvRes1						; move result to axisoffs1..3
		std		Y+1, ConvRes2
		std		Y+2, ConvRes3
		rjmp	KEY_ACT_VAL_REC_OUT				; go out
  KEY_ACT_VAL_1_7_RECALC:
		rcall	LCD_HEX_CONV					; convert it to hex and store to ConvRes
		lds		Temp, LcdSave0					; load the sign
		cpi		Temp, 0b00101101				; test if negative
		brne	KEY_ACT_VAL_REC_ABS				; positive value
		clr		Temp
		sub		Temp, ConvRes1					; make it negative
		mov		ConvRes1, Temp
		clr		Temp
		sbc		Temp, ConvRes2
		mov		ConvRes2, Temp
		clr		Temp
		sbc		Temp, ConvRes3
		mov		ConvRes3, Temp
  KEY_ACT_VAL_REC_ABS:
		ldi		XL, LOW(Axis1State)				; point to Axis1State
		ldi		XH, HIGH(Axis1State)
		ldi		YL, LOW(ConvRes1)				; point to ConvRes
		ldi		YH, HIGH(ConvRes1)
		ldi		ZL, LOW(Ax1Byte1)				; point to 1st axis
		ldi		ZH, HIGH(Ax1Byte1)
		mov		Temp, MenuStatus				; use axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsr		Temp							; use original axis count
		add		XL, Temp						; use correct axis state
		lsl		Temp
		lsl		Temp							; multiply by 4
		add		ZL, Temp						; use correct primary axis
		ld		Temp, X							; load axis state
		sbrc	Temp, 0
		rcall	ADD_2_VAL						; do inverse calculation
		sbrs	Temp, 0
		rcall	SUB_2_VAL
		ldi		YL, LOW(Ax1Offs1)				; point to axis 1 offset
		mov		Temp, MenuStatus				; use axis bits of MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 4
		add		YL, Temp						; use the correct offset register
		st		Y, ConvRes1						; move result to axisoffs1
		std		Y+1, ConvRes2
		std		Y+2, ConvRes3
		ld		Temp, X							; load axis state
		sbrs	Temp, 4							; no 2nd axis, calculation finished
		rjmp	KEY_ACT_VAL_REC_OUT				; go out
		andi	Temp, 0b00001100				; use the 2nd axis bits
		ldi		ZL, LOW(Ax1Byte1)				; point to 1st axis
		add		ZL, Temp						; use the correct 2nd axis
		ld		Temp, X							; load axis state
		sbrc	Temp, 1							; inverse calculation
		rcall	ADD_2_VAL_Y
		sbrs	Temp, 1
		rcall	SUB_2_VAL_Y						; store to Y adress
   KEY_ACT_VAL_REC_OUT:
		clr		EditValue						; reset EditValue to 0
		sbrc	MenuStatus, 7					; jump back to loop if set all zero state set
		rjmp	KEY_SET_ALL_0_INNER
		andi	MenuStatus, 0b10111111			; reset menu state
		andi	Status1, 0b11011111				; reset value/chain desition bit
 KEYBOARD_ACTION_GO_OUT:
		ori		Status1, 0b10000000				; set the keyboard reaction finished state
		sei										; enable interrupts
		pop		Temp2
		pop		Temp
		ret

		;********************************* JUMP ROUTINES *********************************	
LCD_MENU_ON_JMP:
		rjmp	LCD_MENU_ON				; use this trick as branch is out of range
		;*********************************************************************************

MAIN_TRANS_LCD:							; send text or values to the display, depending on
												; MenuStatus, Axis..State and AxisError
 		push	Temp
		push	Temp2
		push	ReadByte
		cli										; disable interrupts
		ldi		Temp, DispOn					; switch the cursor off
		rcall	LCD_SEND_CMD
		ldi		Temp, 0b00100000				; load " " sign
		sts		AxisDesc1, Temp					; store to empty places
		sts		AxisDesc3, Temp
		sts		AxisDesc5, Temp
		sts		AxisDesc7, Temp
		cpi		MenuStatus, 0b00000000			; normal state, transfer values
		brne	LCD_MENU_ON_JMP
		ldi		XH, HIGH(Axis1State)			; point to axis 1 state register
		clr		Temp
		mov		ReadByte, Temp					; use ReadByte as axis counter
  MAIN_TRANS_NORMAL_INNER:
		ldi		Temp, 0b00100000				; load " " sign
		sts		AxisDesc4, Temp					; make 2nd axis empty
		sts		AxisDesc6, Temp
		mov		Temp, ReadByte
		cpi		Temp, 0
		brne	MAIN_TRANS_NORMAL_NO_0
		ldi		Temp, Start1Line				; Send Cursor to Beginning of first line
		rjmp	MAIN_TRANS_NORMAL_GO			; go on
  MAIN_TRANS_NORMAL_NO_0:
		cpi		Temp, 1
		brne	MAIN_TRANS_NORMAL_NO_1
		ldi		Temp, Start2Line				; Send Cursor to Beginning of 2nd line
		rjmp	MAIN_TRANS_NORMAL_GO			; go on
  MAIN_TRANS_NORMAL_NO_1:
		cpi		Temp, 2
		brne	MAIN_TRANS_NORMAL_NO_2
		ldi		Temp, Start3Line				; Send Cursor to Beginning of 3rd line
		rjmp	MAIN_TRANS_NORMAL_GO			; go on
  MAIN_TRANS_NORMAL_NO_2:
		ldi		Temp, Start4Line				; Send Cursor to Beginning of 4th line
  MAIN_TRANS_NORMAL_GO:
		rcall	LCD_SEND_CMD					; send the cursor
		ldi		XL, LOW(Axis1State)
		add		XL, ReadByte					; point to correct state register
		ld		Temp, X							; load state register
		sbrc	Temp, 5
		rjmp	MAIN_TRANS_LCD_ERROR			; print error message when bit set
		sbrc	Temp, 0
		ldi		Temp2, 0b00101101				; load "-" sign
		sbrs	Temp, 0
		ldi		Temp2, 0b00100000				; load " " sign
		sts		AxisDesc0, Temp2				; store to 1st letter
		sbrs	Temp, 4							; skip 2nd axis description when clear
		rjmp	MAIN_TRANS_LCD_AFTER_2ND
		sbrc	Temp, 1
		ldi		Temp2, 0b00101101				; load "-" sign
		sbrs	Temp, 1
		ldi		Temp2, 0b00101011				; load "+" sign
		sts		AxisDesc4, Temp2
		andi	Temp, 0b00001100				; only use the 2nd axis bits
		lsr		Temp							; original 2nd axis count
		lsr		Temp
		ldi		Temp2, 0x31						; for ascii conversion
		add		Temp, Temp2
		sts		AxisDesc6, Temp					; store to description
  MAIN_TRANS_LCD_AFTER_2ND:
		ldi		Temp2, 0x31						; for ascii conversion
		add		Temp2, ReadByte					; convert to ascii
		sts		AxisDesc2, Temp2				; store primary axis number
		ldi		Temp2, 8						; print 8 digits
		ldi		ZL, LOW(AxisDesc0)				; point to axis description
		ldi		ZH, HIGH(AxisDesc0)
		rcall	LCD_PRINT_MEM					; print it
		ldi		ZL, LOW(Axis1Lcd0)				; point to 1st axis
		mov		Temp, ReadByte					; read axis counter
		lsl		Temp							; multiply by 8
		lsl		Temp
		lsl		Temp
		add		ZL, Temp						; use the correct register
		ldi		Temp2, 8						; print 8 digits
		rcall	LCD_PRINT_MEM					; print it
		rjmp	MAIN_TRANS_LCD_NEXT				; jump to next axis
  MAIN_TRANS_LCD_ERROR:
		ldi		Temp2, 16						; print error message
		ldi		ZL, LOW(AxErr*2)				; point to "  Axis   Error  "
		ldi		ZH, HIGH(AxErr*2)
		rcall	LCD_PRINT_PM					; print the text
  MAIN_TRANS_LCD_NEXT:
		inc		ReadByte
		sbrs	ReadByte, 2						; run loop until 4 reached
		rjmp	MAIN_TRANS_NORMAL_INNER			; go to next step
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_ON:
		dec		CntScaler						; increment the display move timer
		brne	LCD_MENU_ON_NO_UPCOUNT
		inc		DispNum							; increment display counter	after 255 cycles
		ldi		Temp, 80
		mov		CntScaler, Temp					; 3s change rate
  LCD_MENU_ON_NO_UPCOUNT:
		ldi		Temp, Start1Line				; send cursor to 1st line
		rcall	LCD_SEND_CMD
		mov		Temp, MenuStatus
		andi	Temp, 0b00000110				; use axis bytes of MenuStatus
		lsr		Temp							; use original axis counter
		ldi		Temp2, 0x31						; for ascii conversion
		add		Temp2, Temp
		sts		AxisDesc2, Temp2
		ldi		XL, LOW(Axis1State)
		ldi		XH, HIGH(Axis1State)			; load axis 1 state
		add		XL, Temp						; use correct axis state register
		ldi		Temp, 0b00100000				; load " " sign
		sts		AxisDesc4, Temp					; make 2nd axis empty
		sts		AxisDesc6, Temp
		ld		Temp, X							; load axis state
		sbrc	Temp, 5
		rjmp	MAIN_TRANS_LCD_MENU_ERROR		; print error message when bit set
		sbrc	Temp, 0
		ldi		Temp2, 0b00101101				; load "-" sign
		sbrs	Temp, 0
		ldi		Temp2, 0b00100000				; load " " sign
		sts		AxisDesc0, Temp2				; store to 1st letter
		sbrs	Temp, 4							; skip 2nd axis description when clear
		rjmp	MAIN_TRANS_LCD_MENU_AFTER_2ND
		sbrc	Temp, 1
		ldi		Temp2, 0b00101101				; load "-" sign
		sbrs	Temp, 1
		ldi		Temp2, 0b00101011				; load "+" sign
		sts		AxisDesc4, Temp2
		andi	Temp, 0b00001100				; only use the 2nd axis bits
		lsr		Temp
		lsr		Temp
		ldi		Temp2, 0x31						; for ascii conversion
		add		Temp, Temp2
		sts		AxisDesc6, Temp					; store to description
  MAIN_TRANS_LCD_MENU_AFTER_2ND:
		ldi		Temp2, 8						; print 8 digits
		ldi		ZH, HIGH(AxisDesc0)
		ldi		ZL, LOW(AxisDesc0)				; point to axis description
		rcall	LCD_PRINT_MEM					; print it
		ldi		ZL, LOW(Axis1Lcd0)				; point to 1st axis
		mov		Temp, MenuStatus
		andi	Temp, 0b00000110
		lsl		Temp							; multiply by 8
		lsl		Temp
		add		ZL, Temp						; use the correct register
		ldi		Temp2, 8						; print 8 digits
		rcall	LCD_PRINT_MEM					; print it
		rjmp	MAIN_TRANS_LCD_MENU_NEXT		; jump to next axis
  MAIN_TRANS_LCD_MENU_ERROR:
		ldi		Temp2, 16						; print error message
		ldi		ZL, LOW(AxErr*2)				; point to "  Axis   Error  "
		ldi		ZH, HIGH(AxErr*2)
		rcall	LCD_PRINT_PM					; print the text
  MAIN_TRANS_LCD_MENU_NEXT:
		ldi		Temp, Start4Line				; send cursor to 4th line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; print the last line
		ldi		ZL, LOW(Main3rdLine*2)			; this is the escape text
		ldi		ZH, HIGH(Main3rdLine*2)
		rcall	LCD_PRINT_PM					; print it
		sbrs	MenuStatus, 6					; print modified value if value input active
		rjmp	LCD_MENU_DESC_NO_VAL
		ldi		Temp, Mid1Line					; send cursor back to middle of 1st line
		rcall	LCD_SEND_CMD
		ldi		ZL, LOW(LcdSave0)
		ldi		ZH, HIGH(LcdSave0)				; send the modified value to display
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM
		ldi		Temp, Start2Line
		rcall	LCD_SEND_CMD
		sbrs	Status1, 5						; test if value/chain menu active
		rjmp	LCD_MENU_DESC_VAL				; no, normal set value menu
		ldi		ZL, LOW(DecChainVal1stLine*2)	; yes, print the desition menu
		ldi		ZH, HIGH(DecChainVal1stLine*2)
		ldi		Temp2, 16
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16
		ldi		ZL, LOW(DecChainVal2ndLine*2)	; yes, print the desition menu
		ldi		ZH, HIGH(DecChainVal2ndLine*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_DESC_VAL:
		cpi		DispNum, 1
		brlo	LCD_MENU_DESC_VAL_0
		breq	LCD_MENU_DESC_VAL_1				; DispNum=1
		clr		DispNum							; reset DispNum and print part a
  LCD_MENU_DESC_VAL_0:
		ldi		ZL, LOW(SetVal1stLineb*2)		; print 2nd value change routine description
		ldi		ZH, HIGH(SetVal1stLineb*2)
		ldi		Temp2, 16
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(SetVal2ndLineb*2)		; print 3rd value change routine description
		ldi		ZH, HIGH(SetVal2ndLineb*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_DESC_VAL_CURS			; jump to cursor positioning
  LCD_MENU_DESC_VAL_1:
		ldi		ZL, LOW(SetVal1stLinea*2)		; print 2nd value change routine description
		ldi		ZH, HIGH(SetVal1stLinea*2)
		ldi		Temp2, 16
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(SetVal2ndLinea*2)		; print 3rd value change routine description
		ldi		ZH, HIGH(SetVal2ndLinea*2)
		rcall	LCD_PRINT_PM					; print it
  LCD_MENU_DESC_VAL_CURS:
		ldi		Temp, CursLargeBlink			; make the cursor blinking
		rcall	LCD_SEND_CMD
		ldi		Temp, Mid1Line					; load cursor to 1st value digit
		add		Temp, EditValue					; shift the cursor to the edit digit
		rcall	LCD_SEND_CMD
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_DESC_NO_VAL:
		ldi		Temp, Start2Line				; send cursor to 2nd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		Temp, 0b00001000
		cp		MenuStatus, Temp
		brsh	LCD_MENU_NO_MAIN_JMP
		cpi		DispNum, 0
		breq	LCD_MENU_MAIN_0
		cpi		DispNum, 5						; 5 different texts
		brlo	LCD_MENU_MAIN_NO_5
		clr		DispNum
  LCD_MENU_MAIN_0:
		ldi		ZL, LOW(Main1stLinee*2)			; print 2nd main routine description
		ldi		ZH, HIGH(Main1stLinee*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(Clear16*2)				; print 3rd main routine description
		ldi		ZH, HIGH(Clear16*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out

		;********************************* JUMP ROUTINES *********************************	
 LCD_MENU_NO_MAIN_JMP:
		rjmp	LCD_MENU_NO_MAIN				; use this trick as branch is out of range
		;*********************************************************************************

  LCD_MENU_MAIN_NO_5:
		cpi		DispNum, 1						; 5 different texts
		brne	LCD_MENU_MAIN_NO_1
		ldi		ZL, LOW(Main1stLinea*2)			; print 2nd main routine description
		ldi		ZH, HIGH(Main1stLinea*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(Main2ndLinea*2)			; print 3rd main routine description
		ldi		ZH, HIGH(Main2ndLinea*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_MAIN_NO_1:
		cpi		DispNum, 2						; 5 different texts
		brne	LCD_MENU_MAIN_NO_2
		ldi		ZL, LOW(Main1stLineb*2)			; print 2nd main routine description
		ldi		ZH, HIGH(Main1stLineb*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(Main2ndLineb*2)			; print 3rd main routine description
		ldi		ZH, HIGH(Main2ndLineb*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_MAIN_NO_2:
		cpi		DispNum, 3						; 5 different text
		brne	LCD_MENU_MAIN_NO_3
		ldi		ZL, LOW(Main1stLinec*2)			; print 2nd main routine description
		ldi		ZH, HIGH(Main1stLinec*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(Main2ndLinea*2)			; print 3rd main routine description
		ldi		ZH, HIGH(Main2ndLinea*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_MAIN_NO_3:
		ldi		ZL, LOW(Main1stLined*2)			; print 2nd main routine description
		ldi		ZH, HIGH(Main1stLined*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(Main2ndLined*2)				; print 3rd main routine description
		ldi		ZH, HIGH(Main2ndLined*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_NO_MAIN:
		sbrs	MenuStatus, 3
		rjmp	LCD_MENU_NO_PRIM_DIR
		cpi		DispNum, 1
		breq	LCD_MENU_PRIM_DIR_1
		brlo	LCD_MENU_PRIM_DIR_0
		clr		DispNum
  LCD_MENU_PRIM_DIR_0:
		ldi		ZL, LOW(PrimSign1stLinea*2)		; print 2nd main routine description
		ldi		ZH, HIGH(PrimSign1stLinea*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(PrimSign2ndLinea*2)		; print 3rd main routine description
		ldi		ZH, HIGH(PrimSign2ndLinea*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_PRIM_DIR_1:
		ldi		ZL, LOW(PrimSign1stLineb*2)		; print 2nd main routine description
		ldi		ZH, HIGH(PrimSign1stLineb*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(PrimSign2ndLineb*2)		; print 3rd main routine description
		ldi		ZH, HIGH(PrimSign2ndLineb*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_NO_PRIM_DIR:
		sbrs	MenuStatus, 4
		rjmp	LCD_MENU_NO_2ND_AX
		ldi		ZL, LOW(SecAxChoose1stLine*2)	; print 2nd sec axis routine description
		ldi		ZH, HIGH(SecAxChoose1stLine*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(SecAxChoose2ndLine*2)	; print 3rd 2nd axis routine description
		ldi		ZH, HIGH(SecAxChoose2ndLine*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_NO_2ND_AX:
		sbrs	MenuStatus, 5
		rjmp	LCD_MENU_NO_2ND_AX_DIR
		cpi		DispNum, 1
		brlo	LCD_MENU_2ND_AX_SIGN_0
		breq	LCD_MENU_2ND_AX_SIGN_1
		clr		DispNum
  LCD_MENU_2ND_AX_SIGN_0:
		ldi		ZL, LOW(SecSign1stLinea*2)		; print 2nd sec axis dir routine description
		ldi		ZH, HIGH(SecSign1stLinea*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(PrimSign2ndLinea*2)		; print 3rd 2nd axis dir routine description
		ldi		ZH, HIGH(PrimSign2ndLinea*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_2ND_AX_SIGN_1:
		ldi		ZL, LOW(PrimSign1stLineb*2)		; print 2nd sec axis dir routine description
		ldi		ZH, HIGH(PrimSign1stLineb*2)
		rcall	LCD_PRINT_PM					; print it
		ldi		Temp, Start3Line				; send cursor to 3rd line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 16						; restore 16 signs
		ldi		ZL, LOW(PrimSign2ndLineb*2)		; print 3rd 2nd axis dir routine description
		ldi		ZH, HIGH(PrimSign2ndLineb*2)
		rcall	LCD_PRINT_PM					; print it
		rjmp	LCD_MENU_OUT					; go out
  LCD_MENU_NO_2ND_AX_DIR:						; change value action is active
LCD_MENU_OUT:
		ldi		XL, LOW(Axis1State)
		ldi		XH, HIGH(Axis1State)
		ld		Temp, X
		andi	Temp, 0b11011111				; reset axis error states
		st		X+, Temp
		ld		Temp, X
		andi	Temp, 0b11011111				; reset axis error states
		st		X+, Temp
		ld		Temp, X
		andi	Temp, 0b11011111				; reset axis error states
		st		X+, Temp
		ld		Temp, X
		andi	Temp, 0b11011111				; reset axis error states
		st		X+, Temp
		sei										; enable interrupts
		pop		ReadByte
		pop		Temp2
		pop		Temp
		ret
		
;**************************************** KEYBOARD READ ROUTINE **************************	


KEYBOARD_READ:							; checks for keyboard activity, sets Status1, 6 if activity
												; writes the hex value to KeyValue
												; cleares Status1, 6 after reaction finished and key released
		push	Temp
		push	Temp2
		cli										; disable interrupts
		in		Temp, KEY_PIN					; read the columns
		ldi		Temp2, 0b00001111				; set lower keypad pins to output
		out		KEY_DDR, Temp2
		ldi		Temp2, 0b11110000
		out		KEY_PORT, Temp2
		rcall	DELAY200						; some time for the signals	
		in		Temp2, KEY_PIN					; read the rows
		or		Temp, Temp2						; add them to 8 Bit
		cpse	Temp, OldKey					; verify old and new keys (deprell)
		rjmp	KEYBOARD_CHANGED				; go out if not equal
		sbrc	Status1, 6						; go out if state already set
		rjmp	KEYBOARD_READ_OUT				; start routine only one time
		cpi		Temp, 0xFF						; no key pressed
		breq	KEYBOARD_OUT_JMP				; go out
		rjmp	KEYBOARD_READ_GO_ON
 KEYBOARD_CHANGED_JMP:							; use this trick as branch is out of range
		rjmp	KEYBOARD_CHANGED

		;********************************* JUMP ROUTINES *********************************	
 KEYBOARD_OUT_JMP:
		rjmp	KEYBOARD_READ_OUT				; use this trick as branch is out of range
		;*********************************************************************************

 KEYBOARD_READ_GO_ON:
		ori		Status1, 0b01000000				; set key pressed state
		cpi		Temp, 0b11101110				; key 1 pressed
		brne	KEYBOARD_READ_NO_1
		ldi		Temp2, 0x01
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_1:
		cpi		Temp, 0b11101101				; key 2 pressed
		brne	KEYBOARD_READ_NO_2
		ldi		Temp2, 0x02
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_2:
		cpi		Temp, 0b11101011				; key 3 pressed
		brne	KEYBOARD_READ_NO_3
		ldi		Temp2, 0x03
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_3:
		cpi		Temp, 0b11100111				; key F pressed
		brne	KEYBOARD_READ_NO_F
		ldi		Temp2, 0x0F
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_F:
		cpi		Temp, 0b11011110				; key 4 pressed
		brne	KEYBOARD_READ_NO_4
		ldi		Temp2, 0x04
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_4:
		cpi		Temp, 0b11011101				; key 5 pressed
		brne	KEYBOARD_READ_NO_5
		ldi		Temp2, 0x05
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_5:
		cpi		Temp, 0b11011011				; key 6 pressed
		brne	KEYBOARD_READ_NO_6
		ldi		Temp2, 0x06
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_6:
		cpi		Temp, 0b11010111				; key E pressed
		brne	KEYBOARD_READ_NO_E
		ldi		Temp2, 0x0E
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_E:
		cpi		Temp, 0b10111110				; key 7 pressed
		brne	KEYBOARD_READ_NO_7
		ldi		Temp2, 0x07
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_7:
		cpi		Temp, 0b10111101				; key 8 pressed
		brne	KEYBOARD_READ_NO_8
		ldi		Temp2, 0x08
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_8:
		cpi		Temp, 0b10111011				; key 9 pressed
		brne	KEYBOARD_READ_NO_9
		ldi		Temp2, 0x09
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_9:
		cpi		Temp, 0b10110111				; key D pressed
		brne	KEYBOARD_READ_NO_D
		ldi		Temp2, 0x0D
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_D:
		cpi		Temp, 0b01111110				; key A pressed
		brne	KEYBOARD_READ_NO_A
		ldi		Temp2, 0x0A
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_A:
		cpi		Temp, 0b01111101				; key 0 pressed
		brne	KEYBOARD_READ_NO_0
		ldi		Temp2, 0x00
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_0:
		cpi		Temp, 0b01111011				; key B pressed
		brne	KEYBOARD_READ_NO_B
		ldi		Temp2, 0x0B
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_B:
		cpi		Temp, 0b01110111				; key C pressed
		brne	KEYBOARD_READ_NO_C
		ldi		Temp2, 0x0C
		mov		KeyValue, Temp2
		rjmp	KEYBOARD_READ_OUT
 KEYBOARD_READ_NO_C:
		andi	Status1, 0b10111111				; don't set read state in this case
		rjmp	KEYBOARD_READ_OUT				; >=2 keys pressed, go out
 KEYBOARD_CHANGED:
		sbrc	Status1, 7						; reset only after menu routine finished
		andi	Status1, 0b00111111				; reset key pressed state and menu finished state
 KEYBOARD_READ_OUT:
		mov		OldKey, Temp					; save pressed key state
		ldi		Temp, 0b11110000				; set upper 4 keypad pins to output
		out		KEY_DDR, Temp
		ldi		Temp, 0b00001111
		out		KEY_PORT, Temp
		sei										; enable interrupts
		pop		Temp2
		pop		Temp
		ret

ADD_2_VAL:								; adds 2 3 Byte values, put 1st value address
												; to Y and 2nd value address into Z
												; result is stored in ConvRes1..ConvRes3
		push	Temp
		push	Temp2
		ld		Temp, Y+						; load 1st byte data
		ld		Temp2, Z+
		add		Temp, Temp2						; add it
		mov		ConvRes1, Temp					; store it
		ld		Temp, Y+						; load 2nd byte data
		ld		Temp2, Z+
		adc		Temp, Temp2						; add it
		mov		ConvRes2, Temp					; store it
		ld		Temp, Y							; load 3rd byte data
		ld		Temp2, Z
		adc		Temp, Temp2						; add it
		mov		ConvRes3, Temp					; store it
		pop		Temp2
		pop		Temp
		ret

SUB_2_VAL:								; subtract 2 3 Byte values, put 1st value address
												; to Y and 2nd value address into Z
												; result is stored in ConvRes1..ConvRes3
		push	Temp
		push	Temp2
		ld		Temp, Y+						; load 1st byte data
		ld		Temp2, Z+
		sub		Temp, Temp2						; subtract
		mov		ConvRes1, Temp					; store result
		ld		Temp, Y+						; load 2nd byte data
		ld		Temp2, Z+
		sbc		Temp, Temp2						; subtract
		mov		ConvRes2, Temp					; store result
		ld		Temp, Y							; load 3rd byte data
		ld		Temp2, Z
		sbc		Temp, Temp2						; subtract
		mov		ConvRes3, Temp					; store result
		pop		Temp2
		pop		Temp
		ret

ADD_2_VAL_Y:							; adds 2 3 Byte values, put 1st value address
												; to Y and 2nd value address into Z
												; result is stored to Y
		push	Temp
		push	Temp2
		ld		Temp, Y							; load 1st byte data
		ld		Temp2, Z+
		add		Temp, Temp2						; add it
		st		Y, Temp							; store to Y
		inc		YL								; next dataset
		ld		Temp, Y							; load 2nd byte data
		ld		Temp2, Z+
		adc		Temp, Temp2						; add it
		st		Y, Temp							; store to Y
		inc		YL								; next dataset
		ld		Temp, Y							; load 3rd byte data
		ld		Temp2, Z
		adc		Temp, Temp2						; add it
		st		Y, Temp							; store to Y
		pop		Temp2
		pop		Temp
		ret

SUB_2_VAL_Y:							; subtract 2 3 Byte values, put 1st value address
												; to Y and 2nd value address into Z
												; result is stored to Y
		push	Temp
		push	Temp2
		ld		Temp, Y							; load 1st byte data
		ld		Temp2, Z+
		sub		Temp, Temp2						; subtract
		st		Y, Temp							; store to Y
		inc		YL								; next dataset
		ld		Temp, Y							; load 2nd byte data
		ld		Temp2, Z+
		sbc		Temp, Temp2						; subtract
		st		Y, Temp							; store to Y
		inc		YL								; next dataset
		ld		Temp, Y							; load 3rd byte data
		ld		Temp2, Z
		sbc		Temp, Temp2						; subtract
		st		Y, Temp							; store to Y
		pop		Temp2
		pop		Temp
		ret
	
;*********************** CONVERSION HEX <---> DISPLAY ASCII VALUE ************************

HEX_DEZ_CONV:							; converts value of ConvRes1..3 into 6decimal values
												; result is stored in (Z) highest...(Z+5)
												; lowest digit
		push	Temp
		push	Temp2
		push	NTemp1
		push	NTemp2
		push	NTemp3
		ldi		Temp2, 0x30						; to convert to ascii
		sbrs	ConvRes3, 7
		rjmp	NEX_DEZ_CONV_POS				; number is positive if ConvRes(7)=0
		clr		Temp							; negative number, calculate absolute value
		sub		Temp, ConvRes1
		mov		ConvRes1, Temp
		clr		Temp
		sbc		Temp, ConvRes2
		mov		ConvRes2, Temp
		clr		Temp
		sbc		Temp, ConvRes3
		mov		ConvRes3, Temp
		ldi		Temp, 0b00101101				; load "-" sign to first letter
		rjmp	HEX_DEZ_CONV_ABS
 NEX_DEZ_CONV_POS:
 		ldi		Temp, 0b00100000				; load " " sign to first letter
 HEX_DEZ_CONV_ABS:								; convert the absolute value
 		st		Z+, Temp						; store number sign
		ldi		Temp, 0xA0						; load dez100000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x86
		mov		NTemp2, Temp
		ldi		Temp, 0x01
		mov		NTemp3, Temp
		clr		Temp							; clear decimal value
 HEX_DEZ_CONV_100000_LOOP:
		sub		ConvRes1, NTemp1				; subtract 100000 until 0 reached
		sbc		ConvRes2, NTemp2
		sbc		ConvRes3, NTemp3
		brlo	HEX_DEZ_CONV_STOP_100000
		inc		Temp							; count 1 up when larger then 100000
		rjmp	HEX_DEZ_CONV_100000_LOOP		; back to loop
 HEX_DEZ_CONV_STOP_100000:
		cpi		Temp, 10
		brlo	HEX_DEZ_CONV_NO_OVF
		subi	Temp, 10
		rjmp	HEX_DEZ_CONV_STOP_100000
  HEX_DEZ_CONV_NO_OVF:
 		add		Temp, Temp2						; convert to ascii value
 		st		Z+, Temp						; save 100000er value
		add		ConvRes1, NTemp1				; undo last subtraction
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		ldi		Temp, 0x10						; load dez10000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x27
		mov		NTemp2, Temp
		clr		Temp							; clear decimal value
		mov		NTemp3, Temp					; not required anymore
 HEX_DEZ_CONV_10000_LOOP:
		sub		ConvRes1, NTemp1				; subtract 10000 until 0 reached
		sbc		ConvRes2, NTemp2
		sbc		ConvRes3, NTemp3
		brlo	HEX_DEZ_CONV_STOP_10000
		inc		Temp							; count 1 up when larger then 10000
		rjmp	HEX_DEZ_CONV_10000_LOOP
 HEX_DEZ_CONV_STOP_10000:
		add		Temp, Temp2						; convert to ascii value
 		st		Z+, Temp						; save 10000er value
		add		ConvRes1, NTemp1				; undo last subtraction
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		ldi		Temp, 0xE8						; load dez1000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x03
		mov		NTemp2, Temp
		clr		Temp							; clear decimal value
 HEX_DEZ_CONV_1000_LOOP:
		sub		ConvRes1, NTemp1				; subtract 1000 until 0 reached
		sbc		ConvRes2, NTemp2
		brlo	HEX_DEZ_CONV_STOP_1000
		inc		Temp							; count 1 up when larger then 1000
		rjmp	HEX_DEZ_CONV_1000_LOOP
 HEX_DEZ_CONV_STOP_1000:
		add		Temp, Temp2						; convert to ascii value
 		st		Z+, Temp						; save 1000er value
		add		ConvRes1, NTemp1				; undo last subtraction
		adc		ConvRes2, NTemp2
		ldi		Temp, 0x64						; load dez100 to NTemp1..3
		mov		NTemp1, Temp
		clr		Temp
		mov		NTemp2, Temp					; not required anymore
 HEX_DEZ_CONV_100_LOOP:
		sub		ConvRes1, NTemp1				; subtract 100 until 0 reached
		sbc		ConvRes2, NTemp2
		brlo	HEX_DEZ_CONV_STOP_100
		inc		Temp							; count 1 up when larger then 100
		rjmp	HEX_DEZ_CONV_100_LOOP
 HEX_DEZ_CONV_STOP_100:
		add		Temp, Temp2						; convert to ascii value
 		st		Z+, Temp						; save 100er value
		inc		ZL								; skip Komma sign
		add		ConvRes1, NTemp1				; undo last subtraction
		adc		ConvRes2, NTemp2
		ldi		Temp, 0x0A						; load dez10 to NTemp1..3
		mov		NTemp1, Temp
		clr		Temp
 HEX_DEZ_CONV_10_LOOP:
		sub		ConvRes1, NTemp1				; subtract 10 until 0 reached
		brlo	HEX_DEZ_CONV_STOP_10
		inc		Temp							; count 1 up when larger then 10
		rjmp	HEX_DEZ_CONV_10_LOOP
 HEX_DEZ_CONV_STOP_10:
		add		Temp, Temp2						; convert to ascii value
 		st		Z+, Temp						; save 10er value
		add		ConvRes1, NTemp1				; undo last subtraction
		add		ConvRes1, Temp2					; convert last to ascii
		st		Z+, ConvRes1					; save 1er value
		pop		NTemp3
		pop		NTemp2
		pop		NTemp1
		pop		Temp2
		pop		Temp
		ret

LCD_HEX_CONV:							; convert LcdSave1..6 into HEX number and write
												; to ConvRes
		push	Temp
		push	Temp2
		clr		Temp
		mov		ConvRes1, Temp					; clear ConsRes
		mov		ConvRes2, Temp
		mov		ConvRes3, Temp
		ldi		Temp2, 0x30						; for ascii-dez-conversion
		ldi		Temp, 0xA0						; load dez100000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x86
		mov		NTemp2, Temp
		ldi		Temp, 0x01
		mov		NTemp3, Temp
		lds		Temp, LcdSave1					; load 100000er value
		sub		Temp, Temp2						; convert back to decimal
 LCD_HEX_100000_INNER:
		cpi		Temp, 0
		breq	LCD_HEX_100000_OUT				; go out if 0 reached
		dec		Temp							; decrease decimal value
		add		ConvRes1, NTemp1				; increase hex value by 100000
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		rjmp	LCD_HEX_100000_INNER			; back to loop
 LCD_HEX_100000_OUT:
		ldi		Temp, 0x10						; load dez10000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x27
		mov		NTemp2, Temp
		clr		Temp							; clear decimal value
		mov		NTemp3, Temp
		lds		Temp, LcdSave2					; load 10000er value
		sub		Temp, Temp2						; convert back to decimal
 LCD_HEX_10000_INNER:
		cpi		Temp, 0
		breq	LCD_HEX_10000_OUT				; go out if 0 reached
		dec		Temp							; decrease decimal value
		add		ConvRes1, NTemp1				; increase hex value by 10000
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		rjmp	LCD_HEX_10000_INNER				; back to loop
 LCD_HEX_10000_OUT:
 		ldi		Temp, 0xE8						; load dez1000 to NTemp1..3
		mov		NTemp1, Temp
		ldi		Temp, 0x03
		mov		NTemp2, Temp
		lds		Temp, LcdSave3					; load 1000er value
		sub		Temp, Temp2						; convert back to decimal
 LCD_HEX_1000_INNER:
		cpi		Temp, 0
		breq	LCD_HEX_1000_OUT				; go out if 0 reached
		dec		Temp							; decrease decimal value
		add		ConvRes1, NTemp1				; increase hex value by 1000
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		rjmp	LCD_HEX_1000_INNER				; back to loop
 LCD_HEX_1000_OUT:
		ldi		Temp, 0x64						; load dez100 to NTemp1..3
		mov		NTemp1, Temp
		clr		Temp
		mov		NTemp2, Temp
		lds		Temp, LcdSave4					; load 100er value
		sub		Temp, Temp2						; convert back to decimal
 LCD_HEX_100_INNER:
		cpi		Temp, 0
		breq	LCD_HEX_100_OUT					; go out if 0 reached
		dec		Temp							; decrease decimal value
		add		ConvRes1, NTemp1				; increase hex value by 100
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		rjmp	LCD_HEX_100_INNER				; back to loop
 LCD_HEX_100_OUT:
		ldi		Temp, 0x0A						; load dez10 to NTemp1..3
		mov		NTemp1, Temp
		lds		Temp, LcdSave6					; load 10er value
		sub		Temp, Temp2						; convert back to decimal
 LCD_HEX_10_INNER:
		cpi		Temp, 0
		breq	LCD_HEX_10_OUT					; go out if 0 reached
		dec		Temp							; decrease decimal value
		add		ConvRes1, NTemp1				; increase hex value by 100
		adc		ConvRes2, NTemp2
		adc		ConvRes3, NTemp3
		rjmp	LCD_HEX_10_INNER				; back to loop
 LCD_HEX_10_OUT:
		lds		Temp, LcdSave7					; load 1er value
		sub		Temp, Temp2						; convert back to decimal
		add		ConvRes1, Temp					; just add the last digit
		adc		ConvRes2, NTemp3				; add zero and carry
		adc		ConvRes3, NTemp3
		pop		Temp2
		pop		Temp
		ret
	
; *************************************** DISPLAY ROUTINES *******************************

LCD_PRINT_PM:							; *** LCD_PrintPM: Prints from program memory
												; Put the starting memory location in Z (r31:r30)
												; Put the number of characters to print in Temp2
												; After execution, Z is at the character AFTER
												; the last to be printed and Temp2 is zero.
												; This function will not wrap if the string
												; is bigger than the LCD.
		push	r0								; r0 is used by lpm command
		push	Temp
 LCD_PM_READ: 
		lpm										; load data adressed by Z into r0
    	mov		Temp, r0						; move it to Temp
		rcall	LCD_SEND_CHAR					; send it to display
		adiw	r30, 1							; increase adress pointer
		dec		Temp2							; decrease number of characters to send
		brne	LCD_PM_READ						; skip if all is sent
		pop		Temp
		pop		r0
		ret

LCD_PRINT_MEM:							; *** LCD_PrintMem: Prints from memory.
												; Put the starting memory location in Z (r31:r30)
												; Put the number of characters to print in Temp2
												; After execution, Z is at the character AFTER
												; the last to be printed and Temp2 is zero.
												; This function will not wrap if you the string
												; is bigger than the LCD.
		push	Temp
 LCD_MEM_READ: 
   		ld		Temp, Z+						; load data adressed by Z into Temp
		rcall	LCD_SEND_CHAR					; send it to display
		dec		Temp2							; decrease number of characters to send
		brne	LCD_MEM_READ					; skip if all is sent
		pop		Temp
		ret

LCD_SEND_CHAR:							; *** Send the Contents of Temp to the LCD using 4bit Mode
		push 	Temp
		push 	Temp2
		;rcall	DISP_OUT						; use this to multiplex display data pins
		mov 	Temp2, Temp						; make a backup copy for nibble swapping
		andi 	Temp2, 0b11110000				; only use the upper nibble first
		swap	Temp2							; move it to lower outputs
		out		LCD_PORT, Temp2					; send it
		sbi		LCD_CRTL, LCD_CRTL_RS			; send data
		;rcall	LCD_RS_HIGH						; send data
		rcall 	LCD_PULSEE						; pulse the enable
		;rcall	DELAY775
		andi	Temp, 0b00001111				;use lower nibble of Temp
		in		Temp2, LCD_PIN					; add the rest of the Port Pins
		andi	Temp2, 0b11110000				; only use the other Pins
		or		Temp, Temp2						; and add them to LCD Port
		out		LCD_PORT, Temp					; send it
		rcall 	LCD_PULSEE						; pulse the enable
		rcall	DELAY200
		cbi		LCD_CRTL, LCD_CRTL_RS			; reset strobe signal
		;rcall	LCD_RS_LOW						; reset strobe signal
		;rcall	DISP_IN							; use this to multiplex display data pins
		pop 	Temp2
		pop 	Temp
		ret

LCD_SEND_CMD:							; ************** LCD_SendCmd: Routine to write a command
												; to the instruction register.
												; The value to be written should be stored in Temp.  
												; The value is sent 4 bits at a time.
		push 	Temp
		push	Temp2
		;rcall	DISP_OUT						; use this to multiplex display data pins
		mov 	Temp2, Temp						; Make a backup copy
		andi 	Temp2, 0b11110000				; Only use the upper nibble
		swap	Temp2							; move it to the lower ports
		out		LCD_PORT, Temp2					; Send it
		rcall 	LCD_PULSEE						; Pulse the enable
		;rcall	DELAY775
		andi 	Temp, 0b00001111				; Only use the lower nibble
		out		LCD_PORT, Temp					; Send it
		rcall 	LCD_PULSEE						; Pulse the enable
		rcall	DELAY200
		;rcall	DISP_IN							; use this to multiplex display data pins
		pop		Temp2
		pop 	Temp
		ret

DISP_TRANS:								; ************** Set the Transfer mode
		push	Temp
		;rcall	DISP_OUT						; use this to multiplex display data pins
		ldi		Temp, Trans8Bit					; set the Display in 8bit mode first
		out		LCD_PORT, Temp
		cbi		LCD_CRTL, LCD_CRTL_RS			; hold RS-Pin low
		;rcall	LCD_RS_LOW
		rcall	LCD_PULSEE						; Send
		rcall	DELAY200						; Time for the Display
		cbi		LCD_PORT,0						; Now it's safe to go into 4-bit mode.
		rcall	LCD_PULSEE
		rcall 	DELAY200
		;rcall	DISP_IN							; use this to multiplex display data pins
		pop		Temp
		ret

;LCD_RS_HIGH:						; ************* Set RS-Pin to high
;		sbi		LCD_CRTL, LCD_CRTL_RS
;		ret

;LCD_RS_LOW:							; ************* Set RS-Pin to low
;		cbi		LCD_CRTL, LCD_CRTL_RS
;		ret		

LCD_PULSEE:	
		sbi		LCD_CRTL, LCD_CRTL_E
		nop
		nop
		cbi		LCD_CRTL, LCD_CRTL_E
		ret

;NUMBER_TO_LCD:							; transmit Data of Lcd1 to Lcd8 to the display
;		push	Temp
;		ldi		Temp, Start1Line				; Send Cursor to Beginning of first line
;		rcall	LCD_SEND_CMD
;		rcall	DELAY4ms						; *** Always use this Delay between a command and the Text entry!!
;		pop		Temp
;		ldi		ZH,0
;		ldi		ZL, 20							; Set Pointer to Lcd1
;		ldi		Temp2, 8
;		rcall	LCD_PRINT_MEM
;		mov		Temp, StatusProg
;		cpi		Temp, 0b00000000
;		breq	NUMBER_TO_LCD_OUT
;		rcall	SET_CURSOR_DIGIT
; NUMBER_TO_LCD_OUT:
;		andi	StatusProg, 0b11111101
;		ret

;DISP_OUT:								; *** Set the LCD Data Pins as Output
												; use it to multiplex display data pins
;		push	Temp
;		in		Temp, LCD_DDR					; read the pin directions
;		ori		Temp, 0b00001111				; set lcd data direction
;		out		LCD_DDR, Temp
;		pop		Temp
;		ret

;DISP_IN:								; *** Set the LCD Data Pins as Input
												; use it to multiplex display data pins
;		push	Temp
;		in		Temp, LCD_DDR					; read the pin directions
;		andi	Temp, 0b11110000				; set the lcd data direction
;		out		LCD_DDR, Temp
;		pop		Temp
;		ret
	
; *********************************** DELAY ROUTINES *************************************

DELAY10s:
		push	DelayTemp
		ldi		DelayTemp, 10
 DELAYCOUNT10s:
 		rcall	DELAY1s
 		dec		DelayTemp
		brne	DELAYCOUNT10s
		pop		DelayTemp
		ret

DELAY1s:								; *** provide 3.971.125 MZ delay
		push	DelayTemp
		ldi		DelayTemp, 255
 DELAYCOUNT1s:
		rcall	DELAY4ms
		dec		DelayTemp
		brne	DELAYCOUNT1s
		pop		DelayTemp
		ret

DELAY4ms:
		push	DelayTemp
		ldi		DelayTemp, 20
 DELAYCOUNT4ms:
 		rcall	DELAY775
 		dec		DelayTemp
		brne	DELAYCOUNT4ms
		pop		DelayTemp
		ret

DELAY775:								; *** provide 775 MZ delay including the rcall command
		push	DelayTemp						; 2MZ
		ldi		DelayTemp, 255					; 1MZ
 DELAYCOUNT:
 		dec		DelayTemp						; 1MZ
		brne	DELAYCOUNT						; 1/2MZ
		pop		DelayTemp						; 2MZ
		ret										; 3MZ, rcall 3MZ

DELAY200:
		push	DelayTemp						; 2MZ
		ldi		DelayTemp, 58					; 1MZ
 DELAYCOUNT200:
 		dec		DelayTemp						; 1MZ
		brne	DELAYCOUNT200					; 1/2MZ
		pop		DelayTemp						; 2MZ
		ret										; 3MZ, rcall 3MZ

;***********************bernommene Subroutinen**********************************

;
;SET_CURSOR_DIGIT:						; move the cursor to the digit determined by Digit
;		push	Temp2
;		mov		Temp2, Digit
;		ldi		Temp, Start1Line				; go to beginning of first line
;		rcall	LCD_SEND_CMD
; SET_CURSOR_DIGIT_SHIFT:						; execute this loop Digit times
;		ldi		Temp, ShCursRight				; shift the cursor right
;		rcall	LCD_SEND_CMD
;		dec		Temp2
;		brne	SET_CURSOR_DIGIT_SHIFT
;		pop		Temp2
;		ret
;


;
;EDIT_DIGITS:						; sets the Digit variable to 1, sets the StatusProg
;												; Edit Mode Bits to 01, moves the cursor to
;												; Digit 1 and makes it a small
;												; visible one
;		push	Temp
;		ldi		Temp, 7
;		mov		Digit, Temp						; set Digit to 8
;		andi	StatusProg, 0b11110111			; Set status bits 32 to 01
;		ori		StatusProg, 0b00000100
;		ldi		Temp, CursSmallOn				; make the cursor a small one
;		rcall	LCD_SEND_CMD
;		rcall	SET_CURSOR_DIGIT				; set the cursoe to digit 1
;		pop		Temp
;		ret
;
;EDIT_OUT:										; set StatusProg edit mode bits to 00 and let the
;												; cursor vanish
;		push	Temp
;		andi	StatusProg, 0b11110011
;		ldi		Temp, DispOn
;		rcall	LCD_SEND_CMD
;		pop		Temp
;		ret
;
;SET_VALUE_OUT:							; Sets Statusprog edit mode bits to 01 and change
;												; the cursor into the small one
;		push	Temp
;		andi	StatusProg, 0b11110111
;		ori		StatusProg, 0b00000100
;		ldi		Temp, CursSmallOn
;		rcall	LCD_SEND_CMD
;		rcall	SET_CURSOR_DIGIT
;		pop		Temp
;		ret
;
;SET_DIGIT:								; set StatusProg edit mode bits to 10 and change the
;												; cursor into the small blinking one
;		push	Temp
;		andi	StatusProg, 0b11111011
;		ori		StatusProg, 0b00001000
;		ldi		Temp, CursSmallBlink
;		rcall	LCD_SEND_CMD
;		rcall	SET_CURSOR_DIGIT
;		pop		Temp
;		ret
